// ********************************************************************
// ROOT macro: generate_ce_cs
// Reads EPICS 2017 data contained in EPDL2017.ALL for Compton effect 
// for all chemical elements (Z=1 to 100).
// Generates 100 cross section data files (one file per element) 
// in compliance with the livermore GEANT4 format in the directory, in which these files are placed :
// pairdataEPDL2017  => contains files for pure pair production "comp"
// Authors: Zhuxin LI, Claire MICHELET @CENBG,France
// 2019.12.09
// ********************************************************************

// main method

void generate_bindingEnergy_2017()
{
	gSystem->Exec("mkdir -p epics2017/phot_preliminary/bindingEnergy");
	 
	FILE * fp = fopen("EADL2017.ALL","r");  
	FILE * final = fopen("epics2017/phot_preliminary/bindingEnergy/binding.dat","w");  
	FILE * final1 = fopen("epics2017/phot_preliminary/bindingEnergy/binding2017_comment.dat","w"); 
	// FILE * fp = fopen("test.txt","r");  
    double designator; //energy 
	double bindigEnergy; // cross section
    Int_t ncols = 0;
   
    char str[64]= "";   
	Int_t z = 1;
	
    
    while (1)
    {
        //read one line of 71 characters		
        if(fgets (str, 64, fp)!= NULL ) 
        {

			// for(int i = 0; i<71; ++i)
			// {
				// printf("%d = %d\n", i+1, str[i]);;
			// }
			//starting from this line :
			// "72  0  0  0.0         0.0         0.0         0.0         0.0", 
			// which is typical for compton effect for one element
            if (!strcmp(str,
                        "91913  0  0.0         0.0         0.0         0.0         0.0\r\n"))
            {
				
                cout << "************************ Z=" << z << endl; // atomic number
                
				
                // char filenameFormat[64] = "epics2017/phot/pe-cs-%d.dat"; 
                char filenameFormat[64] = "epics2017/phot_preliminary/bindingEnergy/be-%d.dat"; 
				char filename[64];
				sprintf (filename,filenameFormat,z); //appends z to the file name
                
                cout << filename << endl;
				
                FILE * fp2 = fopen(filename,"w");	
				// variable input will contain all energy-cross section table for one element				
                string input = ""; 
				// to calculate the total number of eneryg-cross section points
				Int_t loop = 0; 
				// to keep the minimum and maximum energy of the points for one element
				double startDesignator, endDesignator;  
				while (1)
                {
					char designator_e [80] = "";  // a temporary variable
					char data[73] = "";					
					// read the data line (energy - coss section) line by line
					// each line is terminated by a \n which interrupts the reading
					fgets (data, 73, fp); 
					//each table line ends by  "           1"   with 1 written at column 72
					if (strcmp(data,
                        "                                                                       1")==0) 
						{
							// we finish reading the energy - cross section data for the element
							// so now we create the two header lines and put them 
							// at the beginning of the table
							cout<<"ending---------"<<data<<endl;
							endDesignator = designator;
							char headers [100]; //a temporary variable
							
							//save the two header lines in the variable headers
							sprintf(headers,"%.9e %.9e %d\n%d\n", startDesignator, endDesignator, loop,loop ); 
							
							fprintf(final,"%s",input.c_str());
							fprintf (final, "%i           %i\n",-1,-1);
							
							fprintf(final1,"%s",input.c_str());
							fprintf (final1, "%i           %i-------------------Z = %d\n",-1,-1, z);
							
							input = headers + input; //concatenate two header lines with data table
							// write the complete data for one element in fp2 file
							fprintf(fp2,"%s",input.c_str());							
							break;
						}
                    // variable data contains one line energy-cross section					
                    ncols =  sscanf(data,"%lf %lf",&designator,&bindigEnergy); // read energy and cross section respectively from the 
															// valable data keeping the same significant numbers
					// MeV*barn
					// bindigEnergy = designator*designator*designator*bindigEnergy; 
					
					
					// save energy-cross section point in the variable designator_e
					sprintf(designator_e,"%.9e %.9e\n",designator,bindigEnergy); 
					
					//sprintf(designator_e,"%.9e %.9e\n",designator,bindigEnergy);
					if (loop == 0) 
					{
						startDesignator = designator;
					}				
					// concatenate energy-cross section one by one
					input = input + designator_e;  
					// counts the number of data points (lines)
					loop++; 
                    
                }
                fclose(fp2);               
                z++;
               
            } 
		}
		else break;
		
	}
	fprintf (final, "%i           %i\n",-2,-2);
	fprintf (final1, "%i           %i\n",-2,-2);
	fclose(final); 
}
