// **************************************************************************
// ROOT macro: plot_total.c
// Plots the figures of total pair production cross section data. 
// There is one figure (now in png format, it can be modified) per element.
// -	cross section data of EPICS2014 (Z: 1-100) 
// -	cross section data of Livermore (Z: 1-99)   
// -	comparison of cross section data between Livermore and EPICS2014 (Z: 1-99) 
// The generated figures are saved in the corresponding directory:
// -	plots/total/comparison
// -	plots/total/epics2017
// -	plots/total/livermore

// Authors: Zhuxin LI @CENBG,France
// 2019.12.09
// **************************************************************************
TString sInt(Int_t number)
  {
    stringstream ss;
    ss << number;
	TString ts = ss.str();
    return ts;
  }
TString sDouble(Double_t number)
  {
    stringstream ss;
	ss << number;
    TString ts = ss.str();
    return ts;
  }
 struct RatioInfo
 {
	 int z;
	 double ratioPositive;
	 double ratioNegative;
	 double designatorPositive;
	 double designatorNegative;	 
	 
 };
  
  
std::vector<RatioInfo> ratioMaxV;

int Results1=0;
int Results2=0;
int Results3=0;
int Results4=0;

int results=0;
int results1=0;
std::vector<int> resultsV;
int results2=0;
int results3=0;
int results4=0;
int results5=0;
std::vector<int> results5V;

string subShellDesignators[62] = {"K(1s1/2)",    "L(2)",     "L1(2s1/2)",  "L23(2p)",   "L2(2p1/2)", 
								  "L3(2p3/2)",   "M(3)",      "M1(3s1/2)", "M23(3p)",   "M2(3p1/2)",
								  "M3(3p3/2)",   "M45(3d)",   "M4(3d3/2)", "M5(3d5/2)", "N(4)",
								  "N1(4s1/2)",   "N23(4p)",   "N2(4p1/2)", "N3(4p3/2)", "N45(4d)", 
								  "N4(4d3/2)",   "N5(4d5/2)", "N67(4f)",   "N6(4f5/2)", "N7(4f7/2)",
								  "O(5)",        "O1(5s1/2)", "O23(5p)",   "O2(5p1/2)", "O3(5p3/2)",
								  "O45(5d)",     "O4(5d3/2)", "O5(5d5/2)", "O67(5f)",   "O6(5f5/2)",
								  "O7(5f7/2)",   "O89(5g)",   "O8(5g7/2)", "O9(5g9/2)", "P(6)",
								  "P1(6s1/2)",   "P23(6p)",   "P2(6p1/2)", "P3(6p3/2)", "P45(6d)",
								  "P4(6d3/2)",   "P5(6d5/2)", "P67(6f)",   "P6(6f5/2)", "P7(6f7/2)", 
								  "P89(6g)",     "P8(6g7/2)", "P9(6g9/2)", "P1011(6h)", "P10(6h9/2)",
								  "P11(6h11/2)", "Q(7)",      "Q1(7s1/2)", "Q23(7p)",   "Q2(7p1/2)", 
								  "Q3(7p3/2)"};




// void plot(int z,int index, int designator, std::vector<double> subShEnergy1, std::vector<double> subShCS1, 
                          // std::vector<double> subShEnergy2, std::vector<double> subShCS2)
void plot(Int_t z, std::vector<double>& x_1, std::vector<double>& y_1, std::vector<double>& x_2, std::vector<double>& y_2,
std::vector<double>& x_3, std::vector<double>& y_3)
{
	
	gROOT->Reset();
		
	TString S = sInt(z); 
	TString title1;
	
	
	// TString indexS = sInt(index);
	title1 = "Binding energy for element Z = " + S;
	
	
	
	int num1 = x_1.size();
	int num2 = x_2.size();
	int num3 = x_3.size();
	

		
		TCanvas* mycanvas = new TCanvas(S,"Conversion of electrons and gammas",800, 50, 1200, 1000);
		
		mycanvas->SetGrid();
		mycanvas->Divide(1,2);
		
		
		
		mycanvas->cd(1);
		gPad->SetGrid();
		// gPad->SetLogx();
		gPad->SetLogy();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
					
		// TGraph *graph1 = new TGraph(num1,x_1,y_1);
		// TGraph *graph2 = new TGraph(num2,x_2,y_2);
		TGraph *graph1 = new TGraph(num1,x_1.data(),y_1.data());
		TGraph *graph2 = new TGraph(num2,x_2.data(),y_2.data());
		TGraph *graph3 = new TGraph(num3,x_3.data(),y_3.data());
		
		auto mg1 = new TMultiGraph();
		
		
		
		//set graph parameters of EPICS2014
		graph1->SetMarkerStyle(20);  // red point
		graph1->SetMarkerSize(1);
		graph1->SetMarkerColor(2);
		//graph1->SetLineWidth(2);
		graph1->SetLineColor(2);	
		// graph1->Draw("PL");
		
		mg1->Add(graph1);	
		
		//set graph parameters of livermore
		graph2->SetMarkerStyle(4);  //blue circle
		graph2->SetMarkerSize(1.5);
		graph2->SetMarkerColor(4);
		graph2->SetLineColor(4);
		 mg1->Add(graph2);
		 
		//set graph parameters of livermore
		graph3->SetMarkerStyle(26);  //blue circle
		graph3->SetMarkerSize(2.);
		graph3->SetMarkerColor(kTeal+4);
		graph3->SetLineColor(kTeal+4);
		// graph3->SetLineWidth(5);
		
		
		
		// graph2->Draw("APL");
	    mg1->Add(graph3);
		
		mg1->Draw("AP");
		mg1->SetTitle(title1);
	
		mg1->GetXaxis()->SetTitle("Subshell designator ");
		mg1->GetYaxis()->SetTitle("Binding Energy (MeV)");
		
		mg1->GetXaxis()->SetTitleSize(0.05);
		mg1->GetYaxis()->SetTitleSize(0.05);
		mg1->GetXaxis()->SetTitleOffset(1.2);
		
		mg1->GetXaxis()->SetLabelSize(0.05);
		mg1->GetYaxis()->SetLabelSize(0.05);
		
		
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
	
	   
		// mg1->GetXaxis()->SetLimits(0,x_1[num1-1]+2);
		
		if(z ==1 || z ==2)
		{
			mg1->SetMaximum(1e-4); 
			mg1->SetMinimum(1e-5);
		}
		
		// // graph2->SetMinimum(1e-35); 
		
		
		TLegend* leg = new TLegend(0.65,0.7,0.9,0.9);
		// leg->SetTextAlign(12);
		leg->SetFillColor(0);	
		leg->SetTextSize(0.04);
		
		
		TString Tnum1 = sInt(num1);		
		TString Tnum2 = sInt(num2);	
		TString Tnum3 = sInt(num3);	
		
		
		TString name1 = "EADL2017 - " + Tnum1 + " subshell(s)" ;
		
		
		TString name2 = "EADL2014 - " + Tnum2 + " subshell(s)" ;		
		TString name3 = "XDB_EADL - " + Tnum3 + " subshell(s)" ;		
		
		leg->AddEntry(graph1,name1);
		leg->AddEntry(graph2,name2);
		leg->AddEntry(graph3,name3);
		leg->Draw();
		
		
		int Count = 0;
		
		std::vector<double> r(num3, 0.);  //epics2017 VS EDB_EADL
		std::vector<double> R(num3, 0.); //epics2014 VS EDB_EADL
		
		
		// if(num1>=num3) Results2++;
		if(num1==num3) Results2++;
		else if(num1>num3){
			cout<<z<<" "<<num1<<" "<<num3<<endl;
			Results4++;
		}
		else{
			cout<<"**********"<<z<<endl;
			Results3++;
		}
		
		for(int i=0; i<num3;++i)
		{
			for(int j=0; j<num1;++j)
			{
				if(x_1[j] == x_3[i])
				{
					if(y_1[j] == y_3[i])
					{
						r[i] = 0;
						R[i] = 0;
						
					}
					else{
						// r[i] = (y_1[j] - y_3[i])/y_3[i]*100;
						// R[i] = (y_2[j] - y_3[i])/y_3[i]*100;
						
						
						r[i] = (y_3[i] - y_1[j])/y_1[j]*100;
						R[i] = (y_3[i] - y_2[j])/y_2[j]*100;
						
					}
					Count++;
					break;			
				}
			}
			
			
			
		}
		
		// for(int i=0;i<num3;i++)
		// {
			// printf("%d %.9e %.9e %.9e\n",i+1, x_3[i], r[i], R[i]);
		// }
		if(Count ==num3) 
		{
			Results1++;
		}
		else
		{
			cout<<"------------------"<<z<<endl;
		}
		
		
	
		
// to calculate the ratio range
		// double maxPositive = 0;
		// double minNegative = 0;
		
		// RatioInfo ratioInfo;
		// ratioInfo.z = z;
		// ratioInfo.designatorPositive = 0.;
		// ratioInfo.designatorNegative = 0.;
		
		// ratioInfo.ratioPositive = 0.;
		// ratioInfo.ratioNegative = 0.;
		
		
		
		// for(int i=0;i<x_1.size();i++)
		// {
			// if(r[i]>maxPositive)
			// {
				// maxPositive = r[i];
				// ratioInfo.ratioPositive = maxPositive;
				// ratioInfo.designatorPositive = x_1[i];
			// }
			// else if(r[i]<minNegative)
			// {
				// minNegative = r[i];
				// ratioInfo.ratioNegative = minNegative;
				// ratioInfo.designatorNegative = x_1[i];
			// }
		// }
		// ratioMaxV.push_back(ratioInfo);
		
		mycanvas->cd(2);
		gPad->SetGrid();
		// gPad->SetLogx();
		
		// gPad->SetTopMargin(0.12);
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetBottomMargin(0.15);
		// gPad->SetGrid();
		// gStyle->SetTitleW(0.9);  //per cent of the pad width
		// gStyle->SetTitleH(0.9);
		// gStyle->SetTitleFontSize(0.9);
		
		// // pad21->SetLeftMargin(0.10); //10%
		
		
		char nameRatio[100] = "Relative difference of XDB_EADL data compared to EADL2017 and EADL2014";	
		
		TGraph *ratio = new TGraph(num3,x_3.data(), r.data());
		TGraph *Ratio = new TGraph(num3,x_3.data(), R.data());
		
		ratio->SetMarkerStyle(26);
		ratio->SetMarkerSize(2);
		ratio->SetMarkerColor(2);
		ratio->SetLineColor(2);
		
		Ratio->SetMarkerStyle(25);
		Ratio->SetMarkerSize(2);
		Ratio->SetMarkerColor(kBlue);
		Ratio->SetLineColor(kBlue);
		
		auto mg2 = new TMultiGraph();
		
		mg2->Add(ratio);
		mg2->Add(Ratio);
		
		mg2->SetTitle(nameRatio);
		mg2->Draw("AP");
		
		mg2->GetXaxis()->SetTitle("Subshell designator ");
		mg2->GetYaxis()->SetTitle("Relative difference (%)");
		
		mg2->GetXaxis()->SetTitleSize(0.05);
		mg2->GetYaxis()->SetTitleSize(0.05);
		mg2->GetXaxis()->SetTitleOffset(1.2);
		
		mg2->GetXaxis()->SetLabelSize(0.05);
		mg2->GetYaxis()->SetLabelSize(0.05);
		
		
		mg2->GetXaxis()->CenterTitle();
		mg2->GetYaxis()->CenterTitle();
		// mg2->GetXaxis()->SetLimits(0,x_1[num1-1]+2);
		
		TLegend* leg1 = new TLegend(0.65,0.7,0.9,0.9);
		
		// leg->SetTextAlign(12);
		// leg1->SetFillColor(0);	
		leg1->SetFillStyle(0);		
		leg1->SetTextSize(0.04);
		
			
		name1 = "Compared to EADL2017";
		name2 = "Compared to EADL2014";
		
			
		
		leg1->AddEntry(ratio,name1);
		leg1->AddEntry(Ratio,name2);		
		leg1->Draw();
		
		
		
		mycanvas->Print("plots/bindingEnergy_comparison/XDB_EADL/be-"+ S + ".pdf");
		// mycanvas->Close();
		// delete mycanvas;
		// mycanvas = NULL;
		
}

void plot_bindingEnergy_XDB_EADL(){
	
	gROOT->Reset();
	
	gSystem->Exec("mkdir -p plots/bindingEnergy_comparison/XDB_EADL");
	// return;
	double d1; 
	double e1; 
	double d2; 
	double e2; 
    Int_t ncols = 0;
	Int_t count = 0;
	
	
	
 	
 	FILE* fp1 = fopen("epics2017/phot_preliminary/bindingEnergy/binding.dat","r"); 
	// FILE* fp1 = fopen("test1.dat","r"); 
	// FILE* fp2 = fopen("test2.dat","r");
	FILE* fp2 = fopen("epics2014/phot_preliminary/bindingEnergy/binding.dat","r");
	FILE* fp3 = fopen("../G4EMLOW7.10/fluor_XDB_EADL/binding.dat","r");

	
	// Int_t z = 1;
	
	std::vector<double> designator_1[101];
	std::vector<double> be_1[101];
	
	
	std::vector<double> designator_2[101];
	std::vector<double> be_2[101];
	
	std::vector<double> designator_3[101];
	std::vector<double> be_3[101];
// this loop is to read binding data
// do not modify************************	
	for(int i=1; i<=100; ++i)  // there are 100 elements to read, i is the atomic number
	{
// do not modify************************		
		
			
	
// becasue fp1 and fp2 have the same line and same subshell designators, so we can read data
// at the same time	
		while(1)
		{
			count++;
			ncols =  fscanf(fp1,"%lf %lf",&d1,&e1);
			// cout<<"yes"<<endl;
			ncols =  fscanf(fp2,"%lf %lf",&d2,&e2);
			
			
			
			if(d1 ==d2 && d1 == -1)
			{
				// cout<<"---ending---"<<i<<" "<<count<<endl;
				// z++;
				break;
			}
			else{
				
				// printf("%d %.9e %.9e %.9e %.9e\n",count, d1, e1, d2, e2);
			 
				designator_1[i].push_back(d1);
				be_1[i].push_back(e1);
				designator_2[i].push_back(d2);
				be_2[i].push_back(e2);
			 
			
			}
			 
			
		}
		
		
		
		
	}
	fclose(fp1);
	fclose(fp2);
	count = 0;
	
// this loop is to read binding data for XDB_EADL
// do not modify************************	
	for(int i=1; i<=100; ++i)  // there are 100 elements to read, i is the atomic number
	{
// do not modify************************		
	
		
		
		while(1)
		{
			count++;
			ncols =  fscanf(fp3,"%lf %lf",&d1,&e1);
		
			
			if(d1 ==e1 && d1 == -1)
			{
				// cout<<"---ending---"<<i<<" "<<count<<endl;
				// z++;
				break;
			}
			else{
				
				// printf("%d %.9e %.9e\n",count, d1, e1);
			 
				designator_3[i].push_back(d1);
				be_3[i].push_back(e1);
				
			
			}
			 
			
		}
		
		
		
		
	}
	fclose(fp3);
	
// do not modify************************

// to plot
	for(int z =26;z<=26; z++)
	{
		plot(z, designator_1[z], be_1[z], designator_2[z],be_2[z], designator_3[z],be_3[z]);
		// for(int i = 0;i<designator_3[z].size(); i++)
		// {
			// printf("%d %.9e %.9e\n",i+1, designator_3[z][i], be_3[z][i]);
		// }
		
		// for(int i = 0;i<designator_1[z].size(); i++)
		// {
			// printf("%d %.9e %.9e\n",i+1, designator_1[z][i], be_1[z][i]);
		// }
	
	}
	
	cout<<"Results1="<<Results1<<endl;
	cout<<"Results2="<<Results2<<endl;
	cout<<"Results3="<<Results3<<endl;
	cout<<"Results4="<<Results4<<endl;
	
	// FILE* out = fopen("ratioMaxMinOfBindingEnergy.txt", "w");
	// std::vector<RatioInfo>::iterator iter;
		
		// double max = (*ratioMaxV.begin()).ratioPositive;
		// int z1 = (*ratioMaxV.begin()).z;
		// double desig1 = (*ratioMaxV.begin()).designatorPositive;
		
		// double min = (*ratioMaxV.begin()).ratioNegative;
		// int z2 = (*ratioMaxV.begin()).z;
		// double desig2 = (*ratioMaxV.begin()).designatorNegative;
		
	
		
		// for(iter = ratioMaxV.begin(); iter != ratioMaxV.end(); iter++)
		// {
			// fprintf(out,"Element = %d,  ratioMax = %.9e%%  subshell_%lf,\n\
	          // ratioMin = %.9e%% subshell_%lf,  \n", 
					    // (*iter).z, (*iter).ratioPositive, (*iter).designatorPositive, 
								   // (*iter).ratioNegative, (*iter).designatorNegative);
			
			// if((*iter).ratioPositive > max)
			// {
				// max = (*iter).ratioPositive;
				// z1 = (*iter).z;
				// desig1= (*iter).designatorPositive;			
				
				
			// }
			// else if((*iter).ratioNegative< min)
			// {
				// min = (*iter).ratioNegative;
				// z2 = (*iter).z;
				// desig2= (*iter).designatorNegative;	
			// }
		// }
		// fprintf(out,"Amongst all the elements, Max and Min:\n\
		// Element = %d, ratioMax = %.9e%%,  subshell_%lf\n\
		// Element = %d, ratioMin = %.9e%%, subshell_%lf\n", 
					    // z1, max,desig1, z2, min, desig2);
		// fclose(out);
}