// **************************************************************************
// ROOT macro: plot_total.c
// Plots the figures of total pair production cross section data. 
// There is one figure (now in png format, it can be modified) per element.
// -	cross section data of EPICS2014 (Z: 1-100) 
// -	cross section data of Livermore (Z: 1-99)   
// -	comparison of cross section data between Livermore and EPICS2014 (Z: 1-99) 
// The generated figures are saved in the corresponding directory:
// -	plots/total/comparison
// -	plots/total/epics2017
// -	plots/total/livermore

// Authors: Zhuxin LI @CENBG,France
// 2019.12.09
// **************************************************************************
TString sInt(Int_t number)
  {
    stringstream ss;
    ss << number;
	TString ts = ss.str();
    return ts;
  }
TString sDouble(Double_t number)
  {
    stringstream ss;
	ss << number;
    TString ts = ss.str();
    return ts;
  }
 	double RatioMax3[100];
	double RatioMax3_x[100];
	double RatioMax4[100];
	double RatioMax4_x[100];
 
  std::vector<double> fCrossSection_x[101];
  std::vector<double> fCrossSection[101];
  
  std::vector<double> fCrossSectionLE_x[101];
  std::vector<double> fCrossSectionLE[101];
  
  std::vector<double> fParamHigh[101];
  std::vector<double> fParamLow[101];
  int fNShells[101];
  
enum DataToPlot
{
	
	Data2014,
	Data2017,
	Data2017_usingExistingParams
	  	 
};
  
struct Info
{
 Info(int Z, int sub, double r1, double r2)
 :z(Z), subshellIndex(sub), ratio1Max(r1), ratio2Max(r2){}
 int z;
 int subshellIndex;
 double ratio1Max; // for low fit
 double ratio2Max; // for high fit
}; 

struct Average
{
 Average(int Z, int sub, double a1, double a2)
 :z(Z), subshellIndex(sub),average1(a1), average2(a2){}
 int z;
 int subshellIndex;
 double average1; // for low fit
 double average2; // for high fit
};
 
std::map<int, double> ratioMax;
  
int results=0;
int results1=0;
std::vector<int> resultsV;
int results2=0;
int results3=0;
int results4=0;
int results5=0;
std::vector<int> results5V;
int results6=0;
std::vector<int> results6V;
int results7=0;
std::vector<int> results7V;
int results8=0;
int results9=0;
	

void plotComparison_average(std::vector< struct Info>& infoVec2, 
std::vector< struct Info>& infoVec3)
{
	gROOT->Reset();
	
	TString title1;
	TString title2;
	title1 = "Average difference of low fit for subshell according to Z";
	title2 = "Average difference of high fit for subshell according to Z";
		
	
	//2***************************************************************
	vector<double> x(100,0);  // combination of Z and index of subshells
	double tempRatio1 =0;
	double tempRatio2 =0;
	vector<double> y_11(100,0.); // average ratio of low fit
	vector<double> y_22(100,0.); // average of high fit
	int tempZ = 0;
	int tempIndex = 0;
	
	
	for(int i =0; i<infoVec2.size(); ++i)
	{
		struct Info info = infoVec2[i];
		// cout<<"i = "<<i<<endl;
		//Z = 1 and 2, only has one subshell
		if(info.z ==1 || info.z ==2)
		{
			x[i] = info.z;
			y_11[i] = abs(infoVec2[i].ratio1Max);
			y_22[i] = abs(infoVec2[i].ratio2Max);
			continue;
		}
		if(info.z!=tempZ)
		{
			cout<<"____"<<tempZ<<" "<<info.z<<endl;
			tempZ = info.z;
			x[tempZ-1] = tempZ;
			 
			if(tempZ>3)
			{
				y_11[tempZ-2] = tempRatio1/(tempIndex+1); 
				cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-2]<<endl;
				
				y_22[tempZ-2] = tempRatio2/(tempIndex+1);
			}
			
			tempRatio1 = abs(info.ratio1Max);
			tempRatio2 = abs(info.ratio2Max);
			tempIndex =info.subshellIndex;
			
		}
		else{
			
			tempIndex =info.subshellIndex;
			tempRatio1 += abs(info.ratio1Max);
			tempRatio2 += abs(info.ratio2Max);
			
			
			
			if(tempZ ==100 && info.subshellIndex ==26)
			{
				y_11[tempZ-1] = tempRatio1/(tempIndex+1);
				y_22[tempZ-1] = tempRatio2/(tempIndex+1);
				cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-1]<<endl;
				
			}
			
		}
		
	}
	for(int i =0; i<y_11.size();++i)
	{
		// printf("2: %.9e %.9e \n",y_11[i], y_22[i]);
	}
   //3***************************************************************
	tempRatio1 =0;
	tempRatio2 =0;
	vector<double> y_111(100,0.); // ratio of low fit
	vector<double> y_222(100,0.); // ratio of high fit
	tempZ = 0;
	tempIndex = 0;
	
	
	for(int i =0; i<infoVec3.size(); ++i)
	{
		struct Info info = infoVec3[i];
		// cout<<"i = "<<i<<endl;
		//Z = 1 and 2, only has one subshell
		if(info.z ==1 || info.z ==2)
		{
			x[i] = info.z;
			y_111[i] = abs(infoVec3[i].ratio1Max);
			y_222[i] = abs(infoVec3[i].ratio2Max);
			continue;
		}
		if(info.z!=tempZ)
		{
			// cout<<"____"<<tempZ<<" "<<info.z<<endl;
			tempZ = info.z;
			x[tempZ-1] = tempZ;
			
			if(tempZ>3)
			{
								
				y_111[tempZ-2] = tempRatio1/(tempIndex+1); 
				cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-2]<<endl;
				
				y_222[tempZ-2] = tempRatio2/(tempIndex+1);
			}
			
			tempRatio1 = abs(info.ratio1Max);
			tempRatio2 = abs(info.ratio2Max);
			tempIndex =info.subshellIndex;
			
			
		}
		else{
			
			tempIndex =info.subshellIndex;
			tempRatio1 += abs(info.ratio1Max);
			tempRatio2 += abs(info.ratio2Max);
			
			
			
			if(tempZ ==100 && info.subshellIndex ==26)
			{
				y_111[tempZ-1] = tempRatio1/(tempIndex+1);
				y_222[tempZ-1] = tempRatio2/(tempIndex+1);
				cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-1]<<endl;
				
			}
			
		}
		
	}
	for(int i =0; i<y_111.size();++i)
	{
		// cout<<x[i]<<" "<<y_111[i]<<" "<<y_222[i]<<endl;
		// printf("3: %.9e %.9e \n",y_111[i], y_222[i]);
	}
	
	
	TCanvas* mycanvas = new TCanvas("g","Conversion of electrons and gammas",800, 50, 1000,800);
		
		mycanvas->SetGrid();
		mycanvas->Divide(1,2);
		//TPad* pad1 = new TPad("pad1","This is pad1",0.05,0.52,0.95,0.97);
		// TPad* pad1 = new TPad("pad1","This is pad1",0.0,0.4,1.0,1.0);
		// TPad* pad2 = new TPad("pad2","This is pad2",0.0,0.0,1.0,0.4);
		// pad1->Draw();
		// pad2->Draw();
		
		// pad2->cd();
		// TPad* pad21 = new TPad("pad21","First subpad of pad2",0.1,0.0,0.5,1.0);
		// TPad* pad22 = new TPad("pad22","Second subpad of pad2",0.5,0.0,0.9,1.0);
		// pad21->Draw();
		// pad22->Draw();
		
		
		mycanvas->cd(1);
		gPad->SetGrid();
		// gPad->SetLogx();
		// gPad->SetLogy();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
					
		// TGraph *graph1 = new TGraph(num1,x_1,y_1);
		// TGraph *graph2 = new TGraph(num2,x_2,y_2);
		// int size = sizeof(RatioMax_x)/sizeof(double);
		// cout<<size<<endl;
		
		auto mg = new TMultiGraph();
		TGraph *graph11 = new TGraph(x.size(),x.data(),y_11.data());		
		TGraph *graph111 = new TGraph(x.size(),x.data(),y_111.data());		
		
		
		
		
		graph11->SetMarkerStyle(22);  // red point
		graph11->SetMarkerSize(1.2);
		graph11->SetMarkerColor(kTeal+4);
		//graph1->SetLineWidth(2);
		graph11->SetLineColor(kTeal+4);	
		mg->Add(graph11);
		
		graph111->SetMarkerStyle(5);  // red point
		graph111->SetMarkerSize(1.3);
		graph111->SetMarkerColor(kRed);
		//graph1->SetLineWidth(2);
		graph111->SetLineColor(kRed);	
		mg->Add(graph111);
		
		
		// // //set graph parameters of livermore
		
		
		mg->Draw("AP");
				
		mg->SetTitle(title1);		
		mg->GetXaxis()->SetTitle("Atomic number Z");
		mg->GetYaxis()->SetTitle("Relative difference (%)");
		
		mg->GetXaxis()->SetTitleSize(0.06);
		mg->GetYaxis()->SetTitleSize(0.06);
		
		mg->GetXaxis()->SetLabelSize(0.05);
		mg->GetYaxis()->SetLabelSize(0.05);
		
		
		mg->GetXaxis()->CenterTitle();
		mg->GetYaxis()->CenterTitle();
		mg->GetXaxis()->SetLimits(0, 101);
		
		TLegend* leg1 = new TLegend(0.5,0.7,0.9,0.9);
		// leg1->SetTextAlign(12);
		// leg1->SetFillColor(0);
		leg1->SetFillStyle(0);		
		leg1->SetTextSize(0.04);
		
		TString rname1 = "EPDL2014 using exact 2014 fitting parameters";
		TString rname11 = "EPDL2017 using exact 2014 fitting parameters";
		TString rname111 = "EPDL2017 using new fitting parameters";
		
		
		leg1->AddEntry(graph11,rname11);
		leg1->AddEntry(graph111,rname111);
		
		//leg1->DrawClone("Same");
		leg1->Draw();
		
		
		
	//******************	
		
		mycanvas->cd(2);
		gPad->SetGrid();
		// gPad->SetLogx();
		// gPad->SetLogy();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
		auto mg1 = new TMultiGraph();			
		TGraph *graph22 = new TGraph(x.size(),x.data(),y_22.data());
		TGraph *graph222 = new TGraph(x.size(),x.data(),y_222.data());
		
		
		
		graph22->SetMarkerStyle(22);  // red point
		graph22->SetMarkerSize(1.2);
		graph22->SetMarkerColor(kTeal+4);
		//graph1->SetLineWidth(2);
		graph22->SetLineColor(kTeal+4);	
		mg1->Add(graph22);
		
		graph222->SetMarkerStyle(5);  // red point
		graph222->SetMarkerSize(1.3);
		graph222->SetMarkerColor(kRed);
		//graph1->SetLineWidth(2);
		graph222->SetLineColor(kRed);	
		mg1->Add(graph222);

		
		mg1->Draw("AP");
		mg1->SetTitle(title2);		
		mg1->GetXaxis()->SetTitle("Atomic number Z");
		mg1->GetYaxis()->SetTitle("Relative difference (%)");
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
		// TLegend* leg2 = new TLegend(0.7,0.7,0.9,0.9);
		// // leg2->SetTextAlign(12);
		// // leg2->SetFillColor(0);
		// leg2->SetFillStyle(0);		
		// leg2->SetTextSize(0.04);
		
		mg1->GetXaxis()->SetTitleSize(0.06);
		mg1->GetYaxis()->SetTitleSize(0.06);
		
		mg1->GetXaxis()->SetLabelSize(0.05);
		mg1->GetYaxis()->SetLabelSize(0.05);
		mg1->GetXaxis()->SetLimits(0, 101);
		
		TLegend* leg2 = new TLegend(0.5,0.7,0.9,0.9);
		// leg2->SetTextAlign(12);
		// leg2->SetFillColor(0);
		leg2->SetFillStyle(0);		
		leg2->SetTextSize(0.04);
		
		
		
		leg2->AddEntry(graph22,rname11);
		leg2->AddEntry(graph222,rname111);
		
		//leg2->DrawClone("Same");
		leg2->Draw();
		
		
		
		
		mycanvas->Print("plots/averageMax2017UsingExistingParamsVS2017UsingNewParams.png");
		// delete mycanvas;
		// mycanvas = NULL;
			// mycanvas->Close();
		
		
	
		
}

// void plotComparison_averageForEachPoint(std::vector< struct Average>& averageVec1,std::vector< struct Average>& averageVec2, std::vector< struct Average>& averageVec3)
// {
	// gROOT->Reset();
	
	// TString title1;
	// TString title2;
	// title1 = "Average difference of low fit for subshell according to Z";
	// title2 = "Average difference of high fit for subshell according to Z";
	// //1***************************************************************	
	// vector<double> y_1(100,0.); // average ratio of low fit
	// vector<double> y_2(100,0.); // average of high fit
	// vector<double> x(100,0);  // combination of Z and index of subshells
	// double tempRatio1 =0;
	// double tempRatio2 =0;
	// int tempZ = 0;
	// int tempIndex = 0;
	
	// for(int i =0; i<averageVec1.size(); ++i)
	// {
		// struct Average average = averageVec1[i];
		// // cout<<"i = "<<i<<endl;
		// //Z = 1 and 2, only has one subshell
		// if(average.z ==1 || average.z ==2)
		// {
			// x[i] = average.z;
			// y_1[i] = abs(averageVec1[i].average1);
			// y_2[i] = abs(averageVec1[i].average2);
			// continue;
		// }
		// if(average.z!=tempZ)
		// {
			// // cout<<"____"<<tempZ<<" "<<average.z<<endl;
			// tempZ = average.z;
			// x[tempZ-1] = tempZ;
			 
			// if(tempZ>3)
			// {
				// y_1[tempZ-2] = tempRatio1/(tempIndex+1); 
				// // cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_1[tempZ-2]<<endl;
				
				// y_2[tempZ-2] = tempRatio2/(tempIndex+1);
			// }
			
			// tempRatio1 = abs(average.average1);
			// tempRatio2 = abs(average.average2);
			// tempIndex =average.subshellIndex;
			
		// }
		// else{
			
			// tempIndex =average.subshellIndex;
			// tempRatio1 += abs(average.average1);
			// tempRatio2 += abs(average.average2);
			
			
			
			// if(tempZ ==100 && average.subshellIndex ==26)
			// {
				// y_1[tempZ-1] = tempRatio1/(tempIndex+1);
				// y_2[tempZ-1] = tempRatio2/(tempIndex+1);
				// // cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_1[tempZ-1]<<endl;
				
			// }
			
		// }
		
	// }
	// for(int i =0; i<y_1.size();++i)
	// {
		// // cout<<x[i]<<" "<<y_1[i]<<" "<<y_2[i]<<endl;
		// printf("1: %.9e %.9e %.9e \n", x[i], y_1[i], y_2[i]);
	// }
	
	// //2***************************************************************
	
	// vector<double> y_11(100,0.); // average ratio of low fit
	// vector<double> y_22(100,0.); // average of high fit
	
	
	
	// for(int i =0; i<averageVec2.size(); ++i)
	// {
		// struct Average average = averageVec2[i];
		// // cout<<"i = "<<i<<endl;
		// //Z = 1 and 2, only has one subshell
		// if(average.z ==1 || average.z ==2)
		// {
			// x[i] = average.z;
			// y_11[i] = abs(averageVec2[i].average1);
			// y_22[i] = abs(averageVec2[i].average2);
			// continue;
		// }
		// if(average.z!=tempZ)
		// {
			// tempZ = average.z;
			// x[tempZ-1] = tempZ;
			 
			// if(tempZ>3)
			// {
				// y_11[tempZ-2] = tempRatio1/(tempIndex+1); 
				// // cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-2]<<endl;
				
				// y_22[tempZ-2] = tempRatio2/(tempIndex+1);
			// }
			
			// tempRatio1 = abs(average.average1);
			// tempRatio2 = abs(average.average2);
			// tempIndex =average.subshellIndex;
			
		// }
		// else{
			
			// tempIndex =average.subshellIndex;
			// tempRatio1 += abs(average.average1);
			// tempRatio2 += abs(average.average2);
			
			
			
			// if(tempZ ==100 && average.subshellIndex ==26)
			// {
				// y_11[tempZ-1] = tempRatio1/(tempIndex+1);
				// y_22[tempZ-1] = tempRatio2/(tempIndex+1);
				// // cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-1]<<endl;
				
			// }
			
		// }
		
	// }
	// for(int i =0; i<y_11.size();++i)
	// {
		// // cout<<x[i]<<" "<<y_11[i]<<" "<<y_22[i]<<endl;
	// }
   // //3***************************************************************
	// tempRatio1 =0;
	// tempRatio2 =0;
	// vector<double> y_111(100,0.); // ratio of low fit
	// vector<double> y_222(100,0.); // ratio of high fit
	// tempZ = 0;
	// tempIndex = 0;
	
	
	// for(int i =0; i<averageVec3.size(); ++i)
	// {
		// struct Average average = averageVec3[i];
		// // cout<<"i = "<<i<<endl;
		// //Z = 1 and 2, only has one subshell
		// if(average.z ==1 || average.z ==2)
		// {
			// x[i] = average.z;
			// y_111[i] = abs(averageVec3[i].average1);
			// y_222[i] = abs(averageVec3[i].average2);
			// continue;
		// }
		// if(average.z!=tempZ)
		// {
			// // cout<<"____"<<tempZ<<" "<<average.z<<endl;
			// tempZ = average.z;
			// x[tempZ-1] = tempZ;
			
			// if(tempZ>3)
			// {
								
				// y_111[tempZ-2] = tempRatio1/(tempIndex+1); 
				// // cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-2]<<endl;
				
				// y_222[tempZ-2] = tempRatio2/(tempIndex+1);
			// }
			
			// tempRatio1 = abs(average.average1);
			// tempRatio2 = abs(average.average2);
			// tempIndex =average.subshellIndex;
			
			
		// }
		// else{
			
			// tempIndex =average.subshellIndex;
			// tempRatio1 += abs(average.average1);
			// tempRatio2 += abs(average.average2);
			
			
			
			// if(tempZ ==100 && average.subshellIndex ==26)
			// {
				// y_111[tempZ-1] = tempRatio1/(tempIndex+1);
				// y_222[tempZ-1] = tempRatio2/(tempIndex+1);
				// // cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-1]<<endl;
				
			// }
			
		// }
		
	// }
	// for(int i =0; i<y_111.size();++i)
	// {
		// // cout<<x[i]<<" "<<y_111[i]<<" "<<y_222[i]<<endl;
		// printf("3: %.9e %.9e %.9e \n",x[i], y_111[i], y_111[i]);
	// }
	
	
	// TCanvas* mycanvas = new TCanvas("g","Conversion of electrons and gammas",800, 50, 1000,800);
		
		// mycanvas->SetGrid();
		// mycanvas->Divide(1,2);
		// //TPad* pad1 = new TPad("pad1","This is pad1",0.05,0.52,0.95,0.97);
		// // TPad* pad1 = new TPad("pad1","This is pad1",0.0,0.4,1.0,1.0);
		// // TPad* pad2 = new TPad("pad2","This is pad2",0.0,0.0,1.0,0.4);
		// // pad1->Draw();
		// // pad2->Draw();
		
		// // pad2->cd();
		// // TPad* pad21 = new TPad("pad21","First subpad of pad2",0.1,0.0,0.5,1.0);
		// // TPad* pad22 = new TPad("pad22","Second subpad of pad2",0.5,0.0,0.9,1.0);
		// // pad21->Draw();
		// // pad22->Draw();
		
		
		// mycanvas->cd(1);
		// gPad->SetGrid();
		// // gPad->SetLogx();
		// // gPad->SetLogy();
		// gPad->SetBottomMargin(0.15);
		// gPad->SetLeftMargin(0.12);
		// // gPad->SetLeftMargin(0.01);
		// //gPad->SetLogy();
					
		// // TGraph *graph1 = new TGraph(num1,x_1,y_1);
		// // TGraph *graph2 = new TGraph(num2,x_2,y_2);
		// // int size = sizeof(RatioMax_x)/sizeof(double);
		// // cout<<size<<endl;
		
		// auto mg = new TMultiGraph();
		// TGraph *graph1 = new TGraph(x.size(),x.data(),y_1.data());		
		// TGraph *graph11 = new TGraph(x.size(),x.data(),y_11.data());		
		// TGraph *graph111 = new TGraph(x.size(),x.data(),y_111.data());		
		
		
		// graph1->SetMarkerStyle(24);  // red point
		// graph1->SetMarkerSize(1);
		// graph1->SetMarkerColor(kBlue);
		// //graph1->SetLineWidth(2);
		// graph1->SetLineColor(kBlue);	
		// mg->Add(graph1);
		
		
		// graph11->SetMarkerStyle(22);  // red point
		// graph11->SetMarkerSize(1.2);
		// graph11->SetMarkerColor(kTeal+4);
		// //graph1->SetLineWidth(2);
		// graph11->SetLineColor(kTeal+4);	
		// // mg->Add(graph11);
		
		// graph111->SetMarkerStyle(5);  // red point
		// graph111->SetMarkerSize(1.3);
		// graph111->SetMarkerColor(kRed);
		// //graph1->SetLineWidth(2);
		// graph111->SetLineColor(kRed);	
		// mg->Add(graph111);
		
		
		// // // //set graph parameters of livermore
		
		
		// mg->Draw("AP");
				
		// mg->SetTitle(title1);		
		// mg->GetXaxis()->SetTitle("Atomic number Z");
		// mg->GetYaxis()->SetTitle("Relative difference (%)");
		
		// mg->GetXaxis()->SetTitleSize(0.06);
		// mg->GetYaxis()->SetTitleSize(0.06);
		
		// mg->GetXaxis()->SetLabelSize(0.05);
		// mg->GetYaxis()->SetLabelSize(0.05);
		
		
		// mg->GetXaxis()->CenterTitle();
		// mg->GetYaxis()->CenterTitle();
		// mg->GetXaxis()->SetLimits(0, 101);
		
		// TLegend* leg1 = new TLegend(0.5,0.7,0.9,0.9);
		// // leg1->SetTextAlign(12);
		// // leg1->SetFillColor(0);
		// leg1->SetFillStyle(0);		
		// leg1->SetTextSize(0.04);
		
		// TString rname1 = "EPDL2014 using existing fitting parameters";
		// TString rname11 = "EPDL2017 using existing fitting parameters";
		// TString rname111 = "EPDL2017 using new fitting parameters";
		
		
		// leg1->AddEntry(graph1,rname1);
		// // leg1->AddEntry(graph11,rname11);
		// leg1->AddEntry(graph111,rname111);
		
		// //leg1->DrawClone("Same");
		// leg1->Draw();
		
		
		
	// //******************	
		
		// mycanvas->cd(2);
		// gPad->SetGrid();
		// // gPad->SetLogx();
		// // gPad->SetLogy();
		// gPad->SetBottomMargin(0.15);
		// gPad->SetLeftMargin(0.12);
		// // gPad->SetLeftMargin(0.01);
		// //gPad->SetLogy();
		// auto mg1 = new TMultiGraph();			
		// TGraph *graph2 = new TGraph(x.size(),x.data(),y_2.data());
		// TGraph *graph22 = new TGraph(x.size(),x.data(),y_22.data());
		// TGraph *graph222 = new TGraph(x.size(),x.data(),y_222.data());
		
		// graph2->SetMarkerStyle(24);  // red point
		// graph2->SetMarkerSize(1);
		// graph2->SetMarkerColor(kBlue);
		// //graph1->SetLineWidth(2);
		// graph2->SetLineColor(kBlue);	
		// mg1->Add(graph2);
		
		// graph22->SetMarkerStyle(22);  // red point
		// graph22->SetMarkerSize(1.2);
		// graph22->SetMarkerColor(kTeal+4);
		// //graph1->SetLineWidth(2);
		// graph22->SetLineColor(kTeal+4);	
		// // mg1->Add(graph22);
		
		// graph222->SetMarkerStyle(5);  // red point
		// graph222->SetMarkerSize(1.3);
		// graph222->SetMarkerColor(kRed);
		// //graph1->SetLineWidth(2);
		// graph222->SetLineColor(kRed);	
		// mg1->Add(graph222);

		
		// mg1->Draw("AP");
		// mg1->SetTitle(title2);		
		// mg1->GetXaxis()->SetTitle("Atomic number Z");
		// mg1->GetYaxis()->SetTitle("Relative difference (%)");
		// mg1->GetXaxis()->CenterTitle();
		// mg1->GetYaxis()->CenterTitle();
		// // TLegend* leg2 = new TLegend(0.7,0.7,0.9,0.9);
		// // // leg2->SetTextAlign(12);
		// // // leg2->SetFillColor(0);
		// // leg2->SetFillStyle(0);		
		// // leg2->SetTextSize(0.04);
		
		// mg1->GetXaxis()->SetTitleSize(0.06);
		// mg1->GetYaxis()->SetTitleSize(0.06);
		
		// mg1->GetXaxis()->SetLabelSize(0.05);
		// mg1->GetYaxis()->SetLabelSize(0.05);
		// mg1->GetXaxis()->SetLimits(0, 101);
		
		// TLegend* leg2 = new TLegend(0.5,0.7,0.9,0.9);
		// // leg2->SetTextAlign(12);
		// // leg2->SetFillColor(0);
		// leg2->SetFillStyle(0);		
		// leg2->SetTextSize(0.04);
		
		
		
		// leg2->AddEntry(graph2,rname1);
		// // leg2->AddEntry(graph22,rname11);
		// leg2->AddEntry(graph222,rname111);
		
		// //leg2->DrawClone("Same");
		// leg2->Draw();
		
		
		
		
		// mycanvas->Print("plots/averageForEachPoint2017UsingExistingParamsVS2017UsingNewParams.png");
		// // delete mycanvas;
		// // mycanvas = NULL;
			// // mycanvas->Close();
		
		
	
		
// }


// for the first publication
void plotComparison_averageForEachPoint(std::vector< struct Average>& averageVec1,std::vector< struct Average>& averageVec2, std::vector< struct Average>& averageVec3)
{
	gROOT->Reset();
	
	TString title1;
	TString title2;
	title1 = "Average difference of low fit for subshell according to Z";
	title2 = "Average difference of high fit for subshell according to Z";
	//1***************************************************************	
	vector<double> y_1(100,0.); // average ratio of low fit
	vector<double> y_2(100,0.); // average of high fit
	vector<double> x(100,0);  // combination of Z and index of subshells
	double tempRatio1 =0;
	double tempRatio2 =0;
	int tempZ = 0;
	int tempIndex = 0;
	
	for(int i =0; i<averageVec1.size(); ++i)
	{
		struct Average average = averageVec1[i];
		// cout<<"i = "<<i<<endl;
		//Z = 1 and 2, only has one subshell
		if(average.z ==1 || average.z ==2)
		{
			x[i] = average.z;
			y_1[i] = abs(averageVec1[i].average1);
			y_2[i] = abs(averageVec1[i].average2);
			continue;
		}
		if(average.z!=tempZ)
		{
			// cout<<"____"<<tempZ<<" "<<average.z<<endl;
			tempZ = average.z;
			x[tempZ-1] = tempZ;
			 
			if(tempZ>3)
			{
				y_1[tempZ-2] = tempRatio1/(tempIndex+1); 
				// cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_1[tempZ-2]<<endl;
				
				y_2[tempZ-2] = tempRatio2/(tempIndex+1);
			}
			
			tempRatio1 = abs(average.average1);
			tempRatio2 = abs(average.average2);
			tempIndex =average.subshellIndex;
			
		}
		else{
			
			tempIndex =average.subshellIndex;
			tempRatio1 += abs(average.average1);
			tempRatio2 += abs(average.average2);
			
			
			
			if(tempZ ==100 && average.subshellIndex ==26)
			{
				y_1[tempZ-1] = tempRatio1/(tempIndex+1);
				y_2[tempZ-1] = tempRatio2/(tempIndex+1);
				// cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_1[tempZ-1]<<endl;
				
			}
			
		}
		
	}
	for(int i =0; i<y_1.size();++i)
	{
		// cout<<x[i]<<" "<<y_1[i]<<" "<<y_2[i]<<endl;
		printf("1: %.9e %.9e %.9e \n", x[i], y_1[i], y_2[i]);
	}
	
	//2***************************************************************
	
	vector<double> y_11(100,0.); // average ratio of low fit
	vector<double> y_22(100,0.); // average of high fit
	
	
	
	for(int i =0; i<averageVec2.size(); ++i)
	{
		struct Average average = averageVec2[i];
		// cout<<"i = "<<i<<endl;
		//Z = 1 and 2, only has one subshell
		if(average.z ==1 || average.z ==2)
		{
			x[i] = average.z;
			y_11[i] = abs(averageVec2[i].average1);
			y_22[i] = abs(averageVec2[i].average2);
			continue;
		}
		if(average.z!=tempZ)
		{
			tempZ = average.z;
			x[tempZ-1] = tempZ;
			 
			if(tempZ>3)
			{
				y_11[tempZ-2] = tempRatio1/(tempIndex+1); 
				// cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-2]<<endl;
				
				y_22[tempZ-2] = tempRatio2/(tempIndex+1);
			}
			
			tempRatio1 = abs(average.average1);
			tempRatio2 = abs(average.average2);
			tempIndex =average.subshellIndex;
			
		}
		else{
			
			tempIndex =average.subshellIndex;
			tempRatio1 += abs(average.average1);
			tempRatio2 += abs(average.average2);
			
			
			
			if(tempZ ==100 && average.subshellIndex ==26)
			{
				y_11[tempZ-1] = tempRatio1/(tempIndex+1);
				y_22[tempZ-1] = tempRatio2/(tempIndex+1);
				// cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_11[tempZ-1]<<endl;
				
			}
			
		}
		
	}
	for(int i =0; i<y_11.size();++i)
	{
		// cout<<x[i]<<" "<<y_11[i]<<" "<<y_22[i]<<endl;
	}
   //3***************************************************************
	tempRatio1 =0;
	tempRatio2 =0;
	vector<double> y_111(100,0.); // ratio of low fit
	vector<double> y_222(100,0.); // ratio of high fit
	tempZ = 0;
	tempIndex = 0;
	
	
	for(int i =0; i<averageVec3.size(); ++i)
	{
		struct Average average = averageVec3[i];
		// cout<<"i = "<<i<<endl;
		//Z = 1 and 2, only has one subshell
		if(average.z ==1 || average.z ==2)
		{
			x[i] = average.z;
			y_111[i] = abs(averageVec3[i].average1);
			y_222[i] = abs(averageVec3[i].average2);
			continue;
		}
		if(average.z!=tempZ)
		{
			// cout<<"____"<<tempZ<<" "<<average.z<<endl;
			tempZ = average.z;
			x[tempZ-1] = tempZ;
			
			if(tempZ>3)
			{
								
				y_111[tempZ-2] = tempRatio1/(tempIndex+1); 
				// cout<<"final Z = "<<tempZ-1<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-2]<<endl;
				
				y_222[tempZ-2] = tempRatio2/(tempIndex+1);
			}
			
			tempRatio1 = abs(average.average1);
			tempRatio2 = abs(average.average2);
			tempIndex =average.subshellIndex;
			
			
		}
		else{
			
			tempIndex =average.subshellIndex;
			tempRatio1 += abs(average.average1);
			tempRatio2 += abs(average.average2);
			
			
			
			if(tempZ ==100 && average.subshellIndex ==26)
			{
				y_111[tempZ-1] = tempRatio1/(tempIndex+1);
				y_222[tempZ-1] = tempRatio2/(tempIndex+1);
				// cout<<"final Z = "<<tempZ<<" total subshells = "<<tempIndex+1<<" averageRatio = "<<y_111[tempZ-1]<<endl;
				
			}
			
		}
		
	}
	for(int i =0; i<y_111.size();++i)
	{
		// cout<<x[i]<<" "<<y_111[i]<<" "<<y_222[i]<<endl;
		printf("3: %.9e %.9e %.9e \n",x[i], y_111[i], y_111[i]);
	}
	
	
	TCanvas* mycanvas = new TCanvas("g","Conversion of electrons and gammas",800, 50, 400,400);
		
		mycanvas->SetGrid();
		//TPad* pad1 = new TPad("pad1","This is pad1",0.05,0.52,0.95,0.97);
		// TPad* pad1 = new TPad("pad1","This is pad1",0.0,0.4,1.0,1.0);
		// TPad* pad2 = new TPad("pad2","This is pad2",0.0,0.0,1.0,0.4);
		// pad1->Draw();
		// pad2->Draw();
		
		// pad2->cd();
		// TPad* pad21 = new TPad("pad21","First subpad of pad2",0.1,0.0,0.5,1.0);
		// TPad* pad22 = new TPad("pad22","Second subpad of pad2",0.5,0.0,0.9,1.0);
		// pad21->Draw();
		// pad22->Draw();
		
		
		gPad->SetGrid();
		// gPad->SetLogx();
		// gPad->SetLogy();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.15);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
					
		// TGraph *graph1 = new TGraph(num1,x_1,y_1);
		// TGraph *graph2 = new TGraph(num2,x_2,y_2);
		// int size = sizeof(RatioMax_x)/sizeof(double);
		// cout<<size<<endl;
		
		auto mg = new TMultiGraph();
		TGraph *graph1 = new TGraph(x.size(),x.data(),y_1.data());		
		TGraph *graph11 = new TGraph(x.size(),x.data(),y_11.data());		
		TGraph *graph111 = new TGraph(x.size(),x.data(),y_111.data());		
		
		
		graph1->SetMarkerStyle(21);  // red point
		graph1->SetMarkerSize(1);
		graph1->SetMarkerColor(kBlue);
		//graph1->SetLineWidth(2);
		graph1->SetLineColor(kBlue);	
		mg->Add(graph1);
		
		
		graph11->SetMarkerStyle(22);  // red point
		graph11->SetMarkerSize(1.2);
		graph11->SetMarkerColor(kTeal+4);
		//graph1->SetLineWidth(2);
		graph11->SetLineColor(kTeal+4);	
		// mg->Add(graph11);
		
		graph111->SetMarkerStyle(24);  // red point
		graph111->SetMarkerSize(1.2);
		graph111->SetMarkerColor(kRed);
		//graph1->SetLineWidth(2);
		graph111->SetLineColor(kRed);	
		mg->Add(graph111);
		
		
		// // //set graph parameters of livermore
		
		
		mg->Draw("AP");
				
		// mg->SetTitle(title1);		
		mg->GetXaxis()->SetTitle("Atomic number Z");
		mg->GetYaxis()->SetTitle("Absolute average difference (%)");
		
		// mg->GetXaxis()->SetTitleSize(0.06);
		// mg->GetYaxis()->SetTitleSize(0.06);
		
		// mg->GetXaxis()->SetLabelSize(0.05);
		// mg->GetYaxis()->SetLabelSize(0.05);
		
		
		mg->GetXaxis()->CenterTitle();
		mg->GetYaxis()->CenterTitle();
		mg->GetXaxis()->SetLimits(0, 101);
		mg->GetXaxis()->SetTitleOffset(1.5);
		mg->GetYaxis()->SetTitleOffset(1.5);
		
		TGaxis *axisX = new TGaxis(0,gPad->GetUymax(),
		101, gPad->GetUymax(),
		mg->GetXaxis()->GetXmin(),
		mg->GetXaxis()->GetXmax(), 510,"U-");
		// axis->SetLineColor(kRed);
		// axis->SetLabelColor(kRed);
		axisX->Draw();
		
		
		TGaxis *axisY = new TGaxis(101,gPad->GetUymin(),
		101,  gPad->GetUymax(),
		mg->GetYaxis()->GetXmin(),
		mg->GetYaxis()->GetXmax(), 510,"U+");
		// axis->SetLineColor(kRed);
		// axis->SetLabelColor(kRed);
		axisY->Draw();
		
		
		
		TLegend* leg1 = new TLegend(0.5,0.7,0.9,0.9);
		// leg1->SetTextAlign(12);
		// leg1->SetFillColor(0);
		leg1->SetFillStyle(0);		
		leg1->SetTextSize(0.04);
		
		TString rname1 = "EPDL2014 using existing fitting parameters";
		TString rname11 = "EPDL2017 using existing fitting parameters";
		TString rname111 = "EPDL2017 using new fitting parameters";
		
		
		leg1->AddEntry(graph1,rname1);
		// leg1->AddEntry(graph11,rname11);
		leg1->AddEntry(graph111,rname111);
		
		//leg1->DrawClone("Same");
		// leg1->Draw();
		
		mycanvas->Print("plots/averageForEachPoint2017UsingExistingParamsVS2017UsingNewParams_low_fit.png");
		
		
		
	//******************	
		
		//*****************	
		TCanvas* mycanvas1 = new TCanvas("S","Conversion of electrons and gammas",800, 50, 400,400);
		
		mycanvas1->SetGrid();
		gPad->SetGrid();
		// gPad->SetLogx();
		// gPad->SetLogy();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.15);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
		auto mg1 = new TMultiGraph();			
		TGraph *graph2 = new TGraph(x.size(),x.data(),y_2.data());
		TGraph *graph22 = new TGraph(x.size(),x.data(),y_22.data());
		TGraph *graph222 = new TGraph(x.size(),x.data(),y_222.data());
		
		graph2->SetMarkerStyle(21);  // red point
		graph2->SetMarkerSize(1);
		graph2->SetMarkerColor(kBlue);
		//graph1->SetLineWidth(2);
		graph2->SetLineColor(kBlue);	
		mg1->Add(graph2);
		
		graph22->SetMarkerStyle(22);  // red point
		graph22->SetMarkerSize(1.2);
		graph22->SetMarkerColor(kTeal+4);
		//graph1->SetLineWidth(2);
		graph22->SetLineColor(kTeal+4);	
		// mg1->Add(graph22);
		
		graph222->SetMarkerStyle(24);  // red point
		graph222->SetMarkerSize(1.2);
		graph222->SetMarkerColor(kRed);
		//graph1->SetLineWidth(2);
		graph222->SetLineColor(kRed);	
		mg1->Add(graph222);

		
		mg1->Draw("AP");
		// mg1->SetTitle(title2);		
		mg1->GetXaxis()->SetTitle("Atomic number Z");
		mg1->GetYaxis()->SetTitle("Absolute average difference (%)");
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
		// TLegend* leg2 = new TLegend(0.7,0.7,0.9,0.9);
		// // leg2->SetTextAlign(12);
		// // leg2->SetFillColor(0);
		// leg2->SetFillStyle(0);		
		// leg2->SetTextSize(0.04);
		
		// mg1->GetXaxis()->SetTitleSize(0.06);
		// mg1->GetYaxis()->SetTitleSize(0.06);
		
		// mg1->GetXaxis()->SetLabelSize(0.05);
		// mg1->GetYaxis()->SetLabelSize(0.05);
		mg1->GetXaxis()->SetLimits(0, 101);
		mg1->GetXaxis()->SetTitleOffset(1.5);
		mg1->GetYaxis()->SetTitleOffset(1.5);
		
		TGaxis *axisX1 = new TGaxis(0,gPad->GetUymax(),
		101, gPad->GetUymax(),
		mg1->GetXaxis()->GetXmin(),
		mg1->GetXaxis()->GetXmax(), 510,"U-");
		// axis->SetLineColor(kRed);
		// axis->SetLabelColor(kRed);
		axisX1->Draw();
		
		
		TGaxis *axisY1 = new TGaxis(101,gPad->GetUymin(),
		101,  gPad->GetUymax(),
		mg1->GetYaxis()->GetXmin(),
		mg1->GetYaxis()->GetXmax(), 510,"U+");
		// axis->SetLineColor(kRed);
		// axis->SetLabelColor(kRed);
		axisY1->Draw();
		
		
		TLegend* leg2 = new TLegend(0.5,0.7,0.9,0.9);
		// leg2->SetTextAlign(12);
		// leg2->SetFillColor(0);
		leg2->SetFillStyle(0);		
		leg2->SetTextSize(0.04);
		
		
		
		leg2->AddEntry(graph2,rname1);
		// leg2->AddEntry(graph22,rname11);
		leg2->AddEntry(graph222,rname111);
		
		//leg2->DrawClone("Same");
		// leg2->Draw();
		
		
		
		
		mycanvas1->Print("plots/averageForEachPoint2017UsingExistingParamsVS2017UsingNewParams_high_fit.png");
		// delete mycanvas;
		// mycanvas = NULL;
			// mycanvas->Close();
		
		
	
		
}


void plot_compare_AverageDifferenceOfSubshellsWithZ_2014_2017(){
	
	gROOT->Reset();	
	gSystem->Exec("mkdir -p plots");
	


	std::vector<struct Average> AverageVec1;
	std::vector<struct Average> AverageVec2;
	std::vector<struct Average> AverageVec3;
	std::vector<double> r_11; // EPICS2017 low fit ratio
	std::vector<double> r_12; // EPICS2017 high fit ratio
	std::vector<double> r_21; // EPICS2014 low fit ratio
	std::vector<double> r_22; // EPICS2014 low fit ratio



	double r1, r2;	
	int z;
	int subIndex;
	
	
	std::ostringstream ost3;
	
	ost3 <<"./AverageDifferenceForTwoFitsOfSubshellsUsingNewParametersFor2017_final.dat"; // this is ratios for EPDL2017 using 2017's fitting parameters
	std::ifstream fin3(ost3.str().c_str());
	for(int i = 0; i<1612;i++) // 1612 lines 
	{
		
		fin3 >> z >>subIndex>> r1>>r2;
		struct Average average(z, subIndex, r1, r2);
		AverageVec1.push_back(average); 
		// cout<<z<<" "<<subIndex<<" "<<r1<<endl;
	}

	fin3.close();
	
	std::ostringstream ost4;
	ost4 <<"AverageDifferenceForTwoFitsOfSubshellsUsingExistingParameters_2017_final.dat"; // this is ratios for EPDL2017 using 2014's fitting parameters
	std::ifstream fin4(ost4.str().c_str());
	for(int i = 0; i<1612;i++) // 1612 lines 
	{
		
		fin4 >> z >>subIndex>> r1>>r2;
		struct Average average(z, subIndex, r1, r2);
		AverageVec2.push_back(average); 
		// cout<<z<<" "<<subIndex<<" "<<r1<<endl;
	}

	fin4.close();



	std::ostringstream ost5;
	ost5 <<"./epics2017-master/fitPhotoElectricEffect/AverageDifferenceForTwoFitsOfSubshellsUsingExistingParametersFor2014_final2.dat"; // this is ratios for EPDL2017 using 2014's fitting parameters
	std::ifstream fin5(ost5.str().c_str());
	for(int i = 0; i<1612;i++) // 1612 lines 
	{
		
		fin5 >> z >>subIndex>> r1>>r2;
		struct Average average(z, subIndex, r1, r2);
		AverageVec3.push_back(average); 
		// cout<<z<<" "<<subIndex<<" "<<r1<<endl;
	}

	fin5.close();
	
	plotComparison_averageForEachPoint(AverageVec3, AverageVec2, AverageVec1);



}