bool IsEqual(double a,double b , double eps,double releps)
{
	if(a==b)
	{
		return true;
	}
		
	if(fabs(a-b)<=releps*fabs(b))
	{
		return true;
	}
	
	if(fabs(a-b)<eps)
	{
      return true;
	}
	
	
	return false;
}
double eps=1e-20;//absolut difference
double releps=1e-10;//relative difference

struct Info
{
 Info(int Z, int sub = 0)
 :z(Z), subshellIndex(sub){}
 int z;
 int subshellIndex;
 double ratio1Max; // for low fit
 double ratio2Max; // for high fit
};
// std::vector<struct Info> infoVec;
struct Average
{
 Average(int Z, int sub = 0)
 :z(Z), subshellIndex(sub){}
 int z;
 int subshellIndex;
 double average1; // for low fit
 double average2; // for high fit
};

std::vector<struct Info> infoVec;
std::vector<struct Average> averageVec; 

TString sInt(Int_t number)
  {
    stringstream ss;
    ss << number;
	TString ts = ss.str();
    return ts;
  }
TString sDouble(Double_t number)
  {
    stringstream ss;
	ss << number;
    TString ts = ss.str();
    return ts;
  }

int Results1=0;
int Results2=0;
int Results3=0;
int Results4=0;
int Results5=0;
int Results6=0;

int results=0;
int results1=0;
std::vector<int> resultsV;
int results2=0;
int results3=0;

int results5=0;
std::vector<int> results5V;

string subShellDesignators[62] = {"K(1s1/2)",    "L(2)",     "L1(2s1/2)",  "L23(2p)",   "L2(2p1/2)", 
								  "L3(2p3/2)",   "M(3)",      "M1(3s1/2)", "M23(3p)",   "M2(3p1/2)",
								  "M3(3p3/2)",   "M45(3d)",   "M4(3d3/2)", "M5(3d5/2)", "N(4)",
								  "N1(4s1/2)",   "N23(4p)",   "N2(4p1/2)", "N3(4p3/2)", "N45(4d)", 
								  "N4(4d3/2)",   "N5(4d5/2)", "N67(4f)",   "N6(4f5/2)", "N7(4f7/2)",
								  "O(5)",        "O1(5s1/2)", "O23(5p)",   "O2(5p1/2)", "O3(5p3/2)",
								  "O45(5d)",     "O4(5d3/2)", "O5(5d5/2)", "O67(5f)",   "O6(5f5/2)",
								  "O7(5f7/2)",   "O89(5g)",   "O8(5g7/2)", "O9(5g9/2)", "P(6)",
								  "P1(6s1/2)",   "P23(6p)",   "P2(6p1/2)", "P3(6p3/2)", "P45(6d)",
								  "P4(6d3/2)",   "P5(6d5/2)", "P67(6f)",   "P6(6f5/2)", "P7(6f7/2)", 
								  "P89(6g)",     "P8(6g7/2)", "P9(6g9/2)", "P1011(6h)", "P10(6h9/2)",
								  "P11(6h11/2)", "Q(7)",      "Q1(7s1/2)", "Q23(7p)",   "Q2(7p1/2)", 
								  "Q3(7p3/2)"};



 
std::vector<double> fParamHigh[101];
std::vector<double> fParamLow[101];
double fLowLimits[101];
double fHighLimits[101];
  
 void ReadData()
{
	
	//**************************************************************
	// read high energy fitting parameters
	std::ostringstream ost1;
	ost1 << "./epics2017-master/fitPhotoElectricEffect/highFittingParams_final2.dat";
	std::ifstream fin1(ost1.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;		
		int subshell_index;
		double P[6];
		fin1 >> z >> subshell_index;
		fParamHigh[z].push_back(subshell_index);
		for(int i = 0; i<6; i++)
		{
			 fin1 >> P[i];
			 fParamHigh[z].push_back(P[i]);
			 
		}	
	 
	}
	
		
   fin1.close();
   // printf("High fit parameters: %.9e\n", fParamHigh[100][188]);
	//**************************************************************
	// read low energy fitting parameters 
	std::ostringstream ost1_low;
	ost1_low << "./epics2017-master/fitPhotoElectricEffect/lowFittingParams_final2.dat";
	std::ifstream fin1_low(ost1_low.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;		
		int subshell_index;
		double P[6];
		fin1_low >> z >> subshell_index;
		fParamLow[z].push_back(subshell_index);
		for(int i = 0; i<6; i++)
		{
			 fin1_low >> P[i];
			 fParamLow[z].push_back(P[i]);
			 
		}		
	 
	}
	fin1_low.close();
	// printf("Low fit parameters: %.9e\n", fParamLow[100][188]);
	
	//**************************************************************		  

	
	
	//**************************************************************
	// read low and high limits 
	std::ostringstream ost2;
	// ost2 << "low_high_limits_final.dat";
	ost2 << "./epics2017-master/fitPhotoElectricEffect/low_high_limits_final2.dat";
	std::ifstream fin2(ost2.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;
		double lowLimit;
		double highLimit;
		fin2 >> z >> lowLimit>>highLimit;
		fLowLimits[z] = lowLimit;
		fHighLimits[z] = highLimit;
	}
	
	// printf("Low limit = %.9e High limit = %.9e\n",fLowLimits[100],fHighLimits[100]);
	fin2.close();
}
  
void plotComparison(int z,int index, int designator, std::vector<double>& x_1, std::vector<double>& y_1)
{
	cout<<endl;
	cout<<"------------Z = "<<z<<" shell_"<<index<<"--------------"<<endl;
	gROOT->Reset();
		
	TString S = sInt(z); 
	
	TString designatorS = subShellDesignators[designator-1]; 
	
	TString indexS = sInt(index);
		
	
	
	int num1 = x_1.size();
	
	// std::vector<int> vec1;
	// for(int j =0;j<num1;++j)
	// {
		// if(y_1[j] == y_1[j-1] && j)
		// {
			// vec1.push_back(j);
		// }
					
	// }
	std::vector<double>  x_23; // fitted total cs energy
	std::vector<double>  x_24; // fitted total cs energy
	
	std::vector<double>  y_23; // fitted total cs
	std::vector<double>  y_24; // fitted total cs
	
	std::vector<double>  r_23; // fitted total cs ratio
	std::vector<double>  r_24; // fitted total cs ratio
	
	double ratioMax3 = 0.;
	double ratioMax4 = 0.;
	
	int idx = index*7 +1;
	// printf("----------%.9e\n",fLowLimits[z]);
	// printf("----------%.9e\n",fHighLimits[z]);
	
	// printf("----------%.9e\n",fParamHigh[z][idx]);
	// printf("----------%.9e\n",fParamHigh[z][idx+1]);
	// printf("----------%.9e\n",fParamHigh[z][idx+2]);
	// printf("----------%.9e\n",fParamHigh[z][idx+3]);
	// printf("----------%.9e\n",fParamHigh[z][idx+4]);
	// printf("----------%.9e\n",fParamHigh[z][idx+5]);
	
	// printf("----------%.9e\n",fParamLow[z][idx]);
	// printf("----------%.9e\n",fParamLow[z][idx+1]);
	// printf("----------%.9e\n",fParamLow[z][idx+2]);
	// printf("----------%.9e\n",fParamLow[z][idx+3]);
	// printf("----------%.9e\n",fParamLow[z][idx+4]);
	// printf("----------%.9e\n",fParamLow[z][idx+5]);
	double sum3  =  0.0;
	double sum4  =  0.0;
	int count3  =  0;
	int count4  =  0;
		
	for(int i=0; i<x_1.size();i++) // EPICS2017
	{
		// printf("%d %.9e %.9e\n",i, x_1[i], y_1[i]);
		double energy = x_1[i];
		double cs_0 = y_1[i];
		double cs;
		double ratio;
		
		
		// energy = std::max(energy, fParamHigh[z][idx-1]);
    
		double x1 = 1.0/energy;
		double x2 = x1*x1;
		double x3 = x2*x1;
		
		
		
		// high energy parameterisation
		if(energy >= fHighLimits[z]) {
			
			double x4 = x2*x2;
			double x5 = x4*x1;
			
			cs = x1*(fParamHigh[z][idx] + x1*fParamHigh[z][idx+1]
					 + x2*fParamHigh[z][idx+2] + x3*fParamHigh[z][idx+3]
					 + x4*fParamHigh[z][idx+4]+ x5*fParamHigh[z][idx+5]);
			ratio = (cs-cs_0)/cs_0 *100;
			
			x_24.push_back(x_1[i]);	
			y_24.push_back(cs);	
			r_24.push_back(ratio);	
			
			if(abs(ratio)>abs(ratioMax4))
			{
				ratioMax4 = ratio;
			}
			sum4+=abs(ratio);
			count4++;
			// printf("%d %.9e %.9e %.9e %.9e\n", i, x_1[i],y_1[i],cs, ratio);
			
		}
		// low energy parameterisation
		else if(energy >= fLowLimits[z]) 
		{
			double x4 = x2*x2;
			double x5 = x4*x1; //this variable usage can probably be optimized
			
			
			cs = x1*(fParamLow[z][idx] + x1*fParamLow[z][idx+1]
			 + x2*fParamLow[z][idx+2] + x3*fParamLow[z][idx+3]
			 + x4*fParamLow[z][idx+4]+ x5*fParamLow[z][idx+5]);
			ratio = (cs-cs_0)/cs_0 *100;
			
			x_23.push_back(x_1[i]);	
			y_23.push_back(cs);
			r_23.push_back(ratio);
			// printf("%d %.9e %.9e %.9e %.9e\n", i, x_1[i],y_1[i],cs, ratio);	
			
			if(abs(ratio)>abs(ratioMax3))
			{
				ratioMax3 = ratio;
			}
			sum3+=abs(ratio);
			count3++;			
		}
		
		

		// struct Average average(z, index);
		
		
		
	}
		struct Info info(z, index);
		struct Average average(z, index);
		info.ratio1Max = ratioMax3;
		info.ratio2Max = ratioMax4;
		
		average.average1 = sum3/count3;
		average.average2 = sum4/count4;
		
		infoVec.push_back(info);
		averageVec.push_back(average);
	
		TCanvas* mycanvas = new TCanvas(S,"Conversion of electrons and gammas",800, 50, 1200, 1000);
		
		mycanvas->SetGrid();
		
		
		
		mycanvas->cd();
		gPad->SetGrid();
		gPad->SetLogx();
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetLeftMargin(0.01);
		//gPad->SetLogy();
					
		// TGraph *graph1 = new TGraph(num1,x_1,y_1);
		// TGraph *graph2 = new TGraph(num2,x_2,y_2);
		TGraph *graph1 = new TGraph(x_23.size(),x_23.data(),r_23.data());
		TGraph *graph2 = new TGraph(x_24.size(),x_24.data(),r_24.data());
		
		auto mg1 = new TMultiGraph();
		
		
		
		//set graph parameters of EPDL2014
		graph1->SetMarkerStyle(4);  // red point
		graph1->SetMarkerSize(0.7);
		graph1->SetMarkerColor(2);
		//graph1->SetLineWidth(2);
		graph1->SetLineColor(2);		
		mg1->Add(graph1);	
		
		//set graph parameters of livermore
		graph2->SetMarkerStyle(4);  //blue circle
		graph2->SetMarkerSize(0.7);
		graph2->SetMarkerColor(4);
		graph2->SetLineColor(4);		
	    mg1->Add(graph2);
		
		mg1->Draw("AP");
		TString title1;
		title1 = "EPDL2017: Relative difference of fitted subshell cross sections for subshell_" + indexS + " " + designatorS + " of element Z = " + S;
	
		mg1->SetTitle(title1);
	
		mg1->GetXaxis()->SetTitle("Energy (MeV)");
		mg1->GetYaxis()->SetTitle("Relative difference (%)");
		
		// mg1->GetXaxis()->SetTitleSize(0.06);
		// mg1->GetYaxis()->SetTitleSize(0.06);
		
		mg1->GetXaxis()->SetTitleOffset(1.3);
		mg1->GetYaxis()->SetTitleOffset(1);
		
		// mg1->GetXaxis()->SetLabelSize(0.05);
		// mg1->GetYaxis()->SetLabelSize(0.05);
		
		
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
	
	   
	
		TLegend* leg1 = new TLegend(0.7,0.7,0.9,0.9);
		// leg1->SetTextAlign(12);
		// leg1->SetFillColor(0);	
		leg1->SetFillStyle(0);
		leg1->SetTextSize(0.04);
		
		
			
		
		
		TString rname3 = "Low fit";
		TString rname4 = "High fit";
		
		leg1->AddEntry(graph1,rname3);
		leg1->AddEntry(graph2,rname4);
		//leg1->DrawClone("Same");
		leg1->Draw();
		
		
		
		
		
		
		
		//***********(begin)for the zooming of the ratio 		
		
		mycanvas->Print("plots/fittedSubshellCSUsingExistingParameters_2017/Fitted-pe-ss-"+ S + "-shell"+ indexS + ".png");
		// mycanvas->Close();
		delete mycanvas;
		mycanvas = NULL;
		
}

void plot_pe_fittedSubshellCSUsingExistingFitParameters_2017(){
	
	gROOT->Reset();
	// create the 2 directories if they do not existif
	gSystem->Exec("mkdir -p plots/fittedSubshellCSUsingExistingParameters_2017");
	ReadData();
// do not modify*****************begin	
//this is to read the binding energy for epics2017 and epics2014,
//the data read will be used to verify if the minimum energy of each subshell
//is equal to the binding energy	
	FILE*fp3 = fopen("epics2017/phot_preliminary/bindingEnergy/binding.dat","r");	
	FILE*fp4 = fopen("epics2014/phot_preliminary/bindingEnergy/binding.dat","r");	
	
	std::vector<int> designator_1[101];
	std::vector<double> be_1[101];
	
	
	std::vector<int> designator_2[101];
	std::vector<double> be_2[101];
	double d1; 
	double e1; 
	double d2; 
	double e2; 
    Int_t Ncols = 0;
	Int_t count = 0;
	
    for(int i=1; i<=100; ++i)  // there are 100 elements
	{
		
		
		while(1)
		{
			count++; // number of line of -1           -1   
			Ncols = fscanf(fp3,"%lf %lf",&d1,&e1);
			// cout<<"yes"<<endl;
			Ncols = fscanf(fp4,"%lf %lf",&d2,&e2);
			
			
			
			if(d1 ==d2 && d1 == -1)
			{
				// cout<<"---ending---Z = "<<i<<" "<<count<<endl;

				break;
			}
			else{
				
				// printf("%d %.9e %.9e %.9e %.9e\n",count, d1, e1, d2, e2);
			 
				designator_1[i].push_back(d1);
				be_1[i].push_back(e1);
				designator_2[i].push_back(d2);
				be_2[i].push_back(e2);
			 
			
			}
			 
			
		}	
		
	}
	fclose(fp3);
	fclose(fp4);
// do not modify*****************end
	double x;
	double y;
	for (Int_t z = 1;z<=100;++z)// number of element Z, from 1 to 100
	{
		
		TString s = sInt(z);
		TString datafile2017= "epics2017/phot_preliminary/pe-ss-cs-" + s + ".dat";
		
		
		ifstream  data_file_2017(datafile2017, ios::in );
        if(data_file_2017.fail()){
            cout << "Error opening data file 2017"  << endl;
        }else {
            cout << "data file " << datafile2017 << " is opened" << endl;
        }
        if(!data_file_2017) break;	
        
		int nb_subshells = be_1[z].size(); // number of subshells for the given element
		vector<double> x_1[nb_subshells]; 
		vector<double> y_1[nb_subshells]; 
		
		vector<double> x_2[nb_subshells]; 
		vector<double> y_2[nb_subshells];
		
		double x;
		double y;
        int subshell_index = -1;
        for (int N = 0; N < 99999; N++)
        {
            data_file_2017 >> x >> y;
			if(x < -1.5) 
			{ 
                break; 
                
            } 
			else if(0.0 == y) 
			{
				subshell_index++;
			}
			else if(x == -1)
			{
				
			}			
			else
			{
				x_1[subshell_index].push_back(x);
				y_1[subshell_index].push_back(y);
				
			}
		
		}
		data_file_2017.close();
		
		
		
		// int j = 3;
		// for(int i = 0; i<x_1[j].size();i++)
		// {
			// printf("%.9e %.9e\n",x_1[j][i], y_1[j][i]);
		// }
		
		for(int i = 0; i<nb_subshells; i++)
		{
			// if(z!=59 || i!=17 )
			// {
				
				// continue;
			// }
			
			plotComparison(z, i, designator_1[z][i], x_1[i], y_1[i]);
			// printf("------%d %d %d\n", z, i,designator_1[z][i]);
		}
		
		
	
	
		
		

		
	}
	
	
	FILE*pFile = fopen("MaxDifferenceForTwoFitsOfSubshellsUsingExistingParameters_2017.dat","w");	
	
	for(int i =0; i<infoVec.size();i++)
	{
		// if(i==0) cout<<"zhuxin"<<infoVec.size()<<" "<<infoVec[i].subshellIndex<<endl;
		fprintf(pFile, "%d %d %.9e %.9e\n", infoVec[i].z, infoVec[i].subshellIndex, infoVec[i].ratio1Max,infoVec[i].ratio2Max);
		// printf("%d %d %.9e %.9e\n", infoVec[i].z, infoVec[i].subshellIndex, infoVec[i].ratio1Max,infoVec[i].ratio2Max);
	}
	fclose(pFile);
	
	FILE*pFile1 = fopen("AverageDifferenceForTwoFitsOfSubshellsUsingExistingParameters_2017.dat","w");	
	
	for(int i =0; i<averageVec.size();i++)
	{
		// if(i==0) cout<<"zhuxin"<<infoVec.size()<<" "<<infoVec[i].subshellIndex<<endl;
		fprintf(pFile1, "%d %d %.9e %.9e\n", averageVec[i].z, averageVec[i].subshellIndex, averageVec[i].average1,averageVec[i].average2);
	}
	fclose(pFile1);
	
	
	cout<<"Results1="<<Results1<<endl;
	cout<<"Results2="<<Results2<<endl;
	cout<<"Results3="<<Results3<<endl;
	cout<<"Results4="<<Results4<<endl;
	cout<<"Results5="<<Results5<<endl;
	cout<<"Results6="<<Results6<<endl;
	
	
	// the following parts are meaningful only for one element******************
	// if runs for 100 elements, they are not indicative
	cout<<"results="<<results<<", results2="<<results2<<", results3="<<results3<<endl;
	cout<<"results1="<<results1<<endl;
	
	cout<<"for the following subshells, all points in EPDL2014 are found in EPDL2017"
	<<endl;
	for(std::vector<int>::iterator it = resultsV.begin(); it!=resultsV.end(); it++)
	{
		cout<<*it<<endl;
	}
	
	
	cout<<"results5="<<results5<<endl;
	cout<<"for the following subshells, it has exceptions due to the precision"<<endl;
	
	for(std::vector<int>::iterator it = results5V.begin(); it!=results5V.end(); it++)
	{
		cout<<*it<<endl;
	}
	
	// the following parts are meaningful only for one element******************
}