// **************************************************************************
// ROOT macro: plot_total.c
// Plots the figures of total pair production cross section data. 
// There is one figure (now in png format, it can be modified) per element.
// -	cross section data of EPICS2014 (Z: 1-100) 
// -	cross section data of Livermore (Z: 1-99)   
// -	comparison of cross section data between Livermore and EPICS2014 (Z: 1-99) 
// The generated figures are saved in the corresponding directory:
// -	plots/total/comparison
// -	plots/total/epics2017
// -	plots/total/livermore

// Authors: Zhuxin LI @CENBG,France
// 2019.12.09
// **************************************************************************
TString sInt(Int_t number)
  {
    stringstream ss;
    ss << number;
	TString ts = ss.str();
    return ts;
  }
TString sDouble(Double_t number)
  {
    stringstream ss;
	ss << number;
    TString ts = ss.str();
    return ts;
  }
 	double RatioMax3[100];
	double RatioMax3_x[100];
	double RatioMax4[100];
	double RatioMax4_x[100];
 
  std::vector<double> fCrossSection_x[101];
  std::vector<double> fCrossSection[101];
  
  std::vector<double> fCrossSectionLE_x[101];
  std::vector<double> fCrossSectionLE[101];
  
  std::vector<double> fParamHigh[101];
  std::vector<double> fParamLow[101];
  int fNShells[101];
  
  
  
  
  void ReadData()
  {
	  
	  for(int z = 1;z<=100;z++)
	  {
		  //************************************************************** 
		  // read total cross section above K shell binding energy
		  double edgeMin, edgeMax;
		  double x, y;
		  int numberOfNodes;		 
		  std::ostringstream ost;
		  ost <<"phot_epics2014/pe-cs-" << z <<".dat";
		  std::ifstream fin(ost.str().c_str());
		  
		  fin >> edgeMin >> edgeMax >> numberOfNodes;
		  fin >>numberOfNodes;
		  
		   // cout<<"ssssssss"<<z<<" "<<numberOfNodes<<endl;
		  
		  fCrossSection_x[z].reserve(numberOfNodes);
		  fCrossSection[z].reserve(numberOfNodes);
		  for(int i = 0; i<numberOfNodes; ++i)
		  {
			  fin >> x >> y;
			  fCrossSection_x[z].push_back(x);
			  fCrossSection[z].push_back(y);
			  
			  
		  }
		  fin.close();
		  //**************************************************************
		  // read high energy fitting parameters
		  std::ostringstream ost1;
		  ost1 << "phot_epics2014/pe-high-" << z <<".dat";
		  std::ifstream fin1(ost1.str().c_str());
		  fin1 >> fNShells[z] >> x;
		  fParamHigh[z].reserve(fNShells[z]*7+1);
		  
		  for(int i = 0; i<fNShells[z]*7+1;i++)
		  {
			  fin1 >> x;
			  fParamHigh[z].push_back(x);
		  }
		   fin1.close();
		  //**************************************************************
		  // read low energy fitting parameters
		  std::ostringstream ost1_low;
		  ost1_low << "phot_epics2014/pe-low-" << z <<".dat";
		  std::ifstream fin1_low(ost1_low.str().c_str());
		  fin1_low >> x >> x;
		  fParamLow[z].reserve(fNShells[z]*7+1);
		  for(int i = 0; i<fNShells[z]*7+1;i++)
		  {
			  fin1_low >> x;
			  fParamLow[z].push_back(x);
		  }
		  fin1_low.close();
		  //**************************************************************		  
		  // read total cross section below K shell binding energy, but above last shell's binding energy
		  std::ostringstream ost3;
          ost3 <<"phot_epics2014/pe-le-cs-" << z <<".dat";
          std::ifstream fin3(ost3.str().c_str());
		  fin3 >> edgeMin >> edgeMax >> numberOfNodes;
		  fin3 >>numberOfNodes;
		  // cout<<"yyyyyyyy"<<z<<" "<<numberOfNodes<<endl;
		  fCrossSectionLE_x[z].reserve(numberOfNodes);
		  fCrossSectionLE[z].reserve(numberOfNodes);
		  double formerEnergy = 0.;
		  for(int i = 0; i<numberOfNodes; ++i)
		  {
			  fin3 >> x >> y;
			  // remove repeated points, and always save the latter point with bigger cross section
			  if(i && x==formerEnergy)
			  {
				  fCrossSectionLE[z][fCrossSectionLE[z].size()-1] = y;
				  continue;
			  }
			  formerEnergy = x;
			  fCrossSectionLE_x[z].push_back(x);
			  fCrossSectionLE[z].push_back(y);
		  }
		  fin3.close();
		  
	  }
  }

std::map<int, double> ratioMax;
  
int results=0;
int results1=0;
std::vector<int> resultsV;
int results2=0;
int results3=0;
int results4=0;
int results5=0;
std::vector<int> results5V;
int results6=0;
std::vector<int> results6V;
int results7=0;
std::vector<int> results7V;
int results8=0;
int results9=0;
	
	
// void plotComparison(Int_t z, char*dir1, char*dir2, std::vector<double>& be_1, std::vector<double>& be_2)
void plotComparison(Int_t z, char*dir1)
{
	
	gROOT->Reset();
		
	TString S = sInt(z); 
	TString title;
	title = "EPDL2017 Photoelectric reaction: fitted (total) CS VS tabulated (total) CS for element Z = " + S;
		
		
	Int_t num1 = 0;  
	// Int_t num2 = 0; 
	
	
	Int_t ncols;
	double e, cs;
	Int_t loop = 0;
	char str[64] = "";
	cout<<"------------element "<<z<<"-------------------------"<<endl;
	FILE * fp1 = fopen(dir1,"r");
	// cout<<"yes"<<endl;
	while(1)  
	{	
		loop++;
		// if loop = 1 we read the first header line
		fgets (str, 64, fp1);
		// cout<<"yes"<<endl;
		// cout<<str<<endl;
		if(loop ==2)
		{
			// now loop = 2, we read the second header line, namely the number of points
			num1 = stoi(str);  
			// cout<<"number of total pair production points:"<<num1<<endl;
			break;
		}		
		
	}
	
	// double x_1[num1];		
	// double y_1[num1]; 
	
	
	std::vector<double>  x_1;
	std::vector<double>  y_1; // tabulated total cs		

	std::vector<double>  y_2; // fitted total cs
	std::vector<double>  r_2; // fitted total cs ratio
	
	std::vector<double>  x_21; // fitted total cs energy
	std::vector<double>  x_22; // fitted total cs energy
	std::vector<double>  x_23; // fitted total cs energy
	std::vector<double>  x_24; // fitted total cs energy
	
	std::vector<double>  y_21; // fitted total cs
	std::vector<double>  y_22; // fitted total cs
	std::vector<double>  y_23; // fitted total cs
	std::vector<double>  y_24; // fitted total cs

	std::vector<double>  r_21; // fitted total cs ratio
	std::vector<double>  r_22; // fitted total cs ratio
	std::vector<double>  r_23; // fitted total cs ratio
	std::vector<double>  r_24; // fitted total cs ratio
	
	
	double formerEnergy = 0.;
	int idx = fNShells[z]*7 - 5;
	for(Int_t i = 0;i<num1;++i)
	{
		// read one line energy cross section
		ncols = fscanf(fp1,"%lf %lf",&e,&cs); 
		
		// x_1[i]=e;   // keep energy
		// y_1[i]=cs;  // keep cross section
		
		//remove the points whose energy is below last shell's binding energy in tabulated data
		// if(e<fParamHigh[z][idx-1]) continue; 
		
		//remove repeated points, and always keep the point with bigger cross section
		if(i && e == formerEnergy)
		{
			// printf("Repeated--%d %.9e %.9e\n", i+1,e, cs);
			y_1[y_1.size()-1] = cs;
			continue;
		}
		
		formerEnergy = e;
		
		x_1.push_back(e);  
		y_1.push_back(cs);
		// printf("--%d %.9e %.9e\n", i+1, e, cs);
		
		
	}
	fclose(fp1);
	cout<<"num1 = "<<num1<<", x_1.size() = "<<x_1.size()<<endl;
	
	// for(int i=0; i<x_1.size();i++) // EPICS2014
	// {
		// printf("**%d %.9e %.9e\n", i+1,x_1[i], y_1[i]);
	// }
	
	// to reverse the vector fCrossSectionLE_x[z] and fCrossSectionLE[z]
	// reverse(fCrossSectionLE_x[z].begin(), fCrossSectionLE_x[z].end());
	// reverse(fCrossSectionLE[z].begin(), fCrossSectionLE[z].end());
	
	bool test1 = true;
	bool test2 = true;
	
	double ratioMax3 = 0.;
	double ratioMax4 = 0.;
	
	for(int i=0; i<x_1.size();i++) // EPICS2017
	{
		
		double energy = x_1[i];
		double cs_0 = y_1[i];
		double cs;
		double ratio;
		
		// int idx = fNShells[z]*7 - 5;
		energy = std::max(energy, fParamHigh[z][idx-1]);
    
		double x1 = 1.0/energy;
		double x2 = x1*x1;
		double x3 = x2*x1;
		
		
		
		// high energy parameterisation
		if(energy >= fParamHigh[z][0]) {
			
			double x4 = x2*x2;
			double x5 = x4*x1;
			
			cs = x1*(fParamHigh[z][idx] + x1*fParamHigh[z][idx+1]
					 + x2*fParamHigh[z][idx+2] + x3*fParamHigh[z][idx+3]
					 + x4*fParamHigh[z][idx+4]+ x5*fParamHigh[z][idx+5]);
			ratio = (cs-cs_0)/cs_0 *100;
			
			x_24.push_back(x_1[i]);	
			y_24.push_back(cs);	
			r_24.push_back(ratio);	
			
			if(abs(ratio)>abs(ratioMax4))
			{
				ratioMax4 = ratio;
			}
			
		}
		// low energy parameterisation
		else if(energy >= fParamLow[z][0]) 
		{
			double x4 = x2*x2;
			double x5 = x4*x1; //this variable usage can probably be optimized
			
			if(z ==25 || z ==26 || z == 27 || z == 28 || z == 29)
			{
				if(energy<fParamHigh[z][1])
				{
					ratio = 0.;
					cs = y_1[i];
					// x_21.push_back(x_1[i]);	
					// y_21.push_back(cs);
					// r_21.push_back(ratio);
					
					x_22.push_back(x_1[i]);	
					y_22.push_back(cs);
					r_22.push_back(ratio);
					// printf("****%.9e %.9e %.9e\n",x_1[i],cs, ratio);
					
				}
				else{
					cs = x1*(fParamLow[z][idx] + x1*fParamLow[z][idx+1]
					 + x2*fParamLow[z][idx+2] + x3*fParamLow[z][idx+3]
					 + x4*fParamLow[z][idx+4]+ x5*fParamLow[z][idx+5]);
					ratio = (cs-cs_0)/cs_0 *100;
					
					x_23.push_back(x_1[i]);	
					y_23.push_back(cs);
					r_23.push_back(ratio);
					// printf("-----%.9e %.9e %.9e %.9e\n",x_1[i],cs, cs_0, ratio);
					
				}
			}
			else{
				cs = x1*(fParamLow[z][idx] + x1*fParamLow[z][idx+1]
					 + x2*fParamLow[z][idx+2] + x3*fParamLow[z][idx+3]
					 + x4*fParamLow[z][idx+4]+ x5*fParamLow[z][idx+5]);
				ratio = (cs-cs_0)/cs_0 *100;
				
				x_23.push_back(x_1[i]);	
				y_23.push_back(cs);
				r_23.push_back(ratio);
				
			}
			
			
			if(abs(ratio)>abs(ratioMax3))
			{
				ratioMax3 = ratio;
			}				
		}
		// Tabulated values above k-shell ionization energy
		else if(energy >= fParamHigh[z][1]) 
		{
			//(begin)version for 2014
			//search for energy in fCrossSection_x[z] vector
			// std::vector<double>::iterator result = find(fCrossSection_x[z].begin(), fCrossSection_x[z].end(), energy);
			// if(result != fCrossSection_x[z].end()) //if energy exist in fCrossSection_x[z]
			// {
				// // get its index, and use its index to get corresponding CS
				// int index = distance(fCrossSection_x[z].begin(),result);
				// cs = x3*fCrossSection[z][index];
				// ratio = (cs-cs_0)/cs_0 *100;
				
				// x_22.push_back(x_1[i]);	
				// y_22.push_back(cs);
				// r_22.push_back(ratio);
				
			// }
			// else
			// {
				// test1 = false;
				// printf("-------%d %.9e\n", i+1, energy);
			// }
			//(end)version for 2014
			ratio = 0.;
			cs = y_1[i];
			x_22.push_back(x_1[i]);	
			y_22.push_back(cs);
			r_22.push_back(ratio);
			
		}
		// Tabulated values below k-shell ionization energy
		else
		{
			
			//(begin)version for 2014
			// std::vector<double>::iterator result = find(fCrossSectionLE_x[z].begin(), fCrossSectionLE_x[z].end(), energy);
			// if(result != fCrossSectionLE_x[z].end())
			// {
				
				// int index = distance(fCrossSectionLE_x[z].begin(),result);
				// cs = x3*fCrossSectionLE[z][index];
				// ratio = (cs-cs_0)/cs_0 *100;
				
				// x_21.push_back(x_1[i]);	
				// y_21.push_back(cs);
				// r_21.push_back(ratio);
				// // printf("---%d %.9e\n", index, energy);
				// // printf("ttt%d %.9e %.9e %.9e\n", i+1,energy, cs, ratio);
			// }
			// else
			// {
				// test2 = false;
				// printf("*******%d %.9e\n", i+1, energy);
			// }
			//(end)version for 2014
			ratio = 0.;
			cs = y_1[i];
			x_21.push_back(x_1[i]);	
			y_21.push_back(cs);
			r_21.push_back(ratio);
			
			
		}
		y_2.push_back(cs);
		r_2.push_back(ratio);
		
		
		
		
	}
	
	RatioMax3[z-1] = ratioMax3;
	RatioMax4[z-1] = ratioMax4;
	
	// return;
	if(!test1)
	{
		results1++;
	}
	if(!test2)
	{
		results2++;
	}
	// cout<<"number of points for 4 functions: "<<y_21.size()<<" "<<y_22.size()<<" "<<y_23.size()<<" "<<y_24.size()<<endl;
	
	// for(int i= 0;i<y_21.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_21[i], y_21[i], r_21[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_22.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_22[i], y_22[i], r_22[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_23.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_23[i], y_23[i], r_23[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_24.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_24[i], y_24[i], r_24[i]);
	// }	
		
	TCanvas* mycanvas = new TCanvas(S,"Conversion of electrons and gammas",800, 50, 1200, 1000);
		
		mycanvas->SetGrid();
		
		gPad->SetLogx();
		gPad->SetGrid();
		
		// gPad->SetTopMargin(0.12);
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetBottomMargin(0.15);
		
		// gStyle->SetTitleW(0.9);  //per cent of the pad width
		// gStyle->SetTitleH(0.9);
		// gStyle->SetTitleFontSize(0.9);
		
		// // pad21->SetLeftMargin(0.10); //10%
		
		
		
		auto mg1 = new TMultiGraph();
		

		TGraph *Ratio23 = new TGraph(x_23.size(),x_23.data(),r_23.data());
		TGraph *Ratio24 = new TGraph(x_24.size(),x_24.data(),r_24.data());
		
		
		Ratio23->SetMarkerStyle(24);  //red point
		Ratio23->SetMarkerSize(0.8);
		Ratio23->SetMarkerColor(kBlue);
		Ratio23->SetLineColor(kBlue);
		mg1->Add(Ratio23);	
		
		
		Ratio24->SetMarkerStyle(24);  //red point
		Ratio24->SetMarkerSize(0.8);
		Ratio24->SetMarkerColor(kRed);
		Ratio24->SetLineColor(kRed);
		mg1->Add(Ratio24);	
		
		
		TString title2 = "EPDL2017: Relative difference of fitted total cross sections using existing fitting parameters for element " + S;
		mg1->SetTitle(title2);
		mg1->Draw("AP");
		
		mg1->GetXaxis()->SetTitle("Energy (MeV)");
		mg1->GetYaxis()->SetTitle("Relative difference (%)");
		
		// mg1->GetXaxis()->SetTitleSize(0.05);
		// mg1->GetYaxis()->SetTitleSize(0.05);
		// mg1->GetXaxis()->SetTitleOffset(1.2);
		
		// mg1->GetXaxis()->SetLabelSize(0.05);
		// mg1->GetYaxis()->SetLabelSize(0.05);
		
		
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
		mg1->GetXaxis()->SetTitleOffset(1.2);
		// mg1->GetXaxis()->SetLimits(1e-6,1e5);
	
		TLegend* leg1 = new TLegend(0.7,0.7,0.9,0.9);
		// leg1->SetTextAlign(12);
		// leg1->SetFillColor(0);	
		leg1->SetFillStyle(0);
		leg1->SetTextSize(0.04);
		
		
			
		
		
		TString rname3 = "Low fit";
		TString rname4 = "High fit";
		
		leg1->AddEntry(Ratio23,rname3);
		leg1->AddEntry(Ratio24,rname4);
		//leg1->DrawClone("Same");
		leg1->Draw();
		
		
		// // TLatex latex;
		// // latex.SetTextSize(0.05);
		// // latex.SetTextAlign(22);
		// // latex.DrawLatexNDC(0.7,0.8,"RD = #frac{EPICS2017-EPICS2014}{EPICS2014} x 100");
		
		
		// // TString Tflag = sInt(k);	
		// // Tflag += " points in comparison";
		// // latex.SetTextColor(2);
		// // latex.DrawLatexNDC(0.7,0.7,Tflag);
		
		
		
		
		// // // TLegend* leg212 =  new TLegend(0.12,0.7,0.3,0.88);
		// // // TString Tnum4 = sInt(num2);
		// // // TString name4 = Tnum4 + " points in comparison";		
		// // // leg212->AddEntry(ratio,name4);
		// // // leg212->SetTextAlign(22);
		// // // leg212->SetFillStyle(0);
		// // // leg212->Draw();
		// // // leg212->SetTextSize(0.05);
		
		
		
		
		// // //***********(begin)for the zoomin of the ratio 
		
		
		
		mycanvas->Print("plots/fittedTotalCSUsingExistingParameters/fittedCS2017-"+ S +".png");
		// delete mycanvas;
		// mycanvas = NULL;
		// mycanvas->Close();
		
		
}



void plot_pe_fittedTotalCSUsingExistingFitParameters_2017(){
	
	gROOT->Reset();	
	gSystem->Exec("mkdir -p plots/fittedTotalCSUsingExistingParameters");
	

	
// // do not modify*****************begin	
// //this is to read the binding energy for epics2017 and epics2014,
// //the data read will be used to verify if the repeated energy  
// //is equal to the binding energy	
	FILE*fp3 = fopen("./epics2017/phot_preliminary/bindingEnergy/binding.dat","r");	
	// FILE*fp4 = fopen("epics2014_zhuxin/bindingenergy/binding.dat","r");	
	
	std::vector<double> designator_1[101];
	std::vector<double> be_1[101];
	
	
	// std::vector<double>* designator_2[101];
	// std::vector<double>* be_2[101];
	double d1; 
	double e1; 
	// double d2; 
	// double e2; 
    Int_t Ncols = 0;
	Int_t count = 0;
	
    for(int i=1; i<=100; ++i)  // there are 100 elements
	{
		
		while(1)
		{


			count++;
			Ncols = fscanf(fp3,"%lf %lf",&d1,&e1);
			// cout<<"yes"<<endl;
			// Ncols = fscanf(fp4,"%lf %lf",&d2,&e2);
			
			
			
			// if(d1 ==d2 && d1 == -1)
			if(d1 == -1)
			{
				// cout<<"---ending---"<<i<<" "<<count<<endl;

				break;
			}
			
			else{
				
				// printf("%d %.9e %.9e %.9e %.9e\n",count, d1, e1, d2, e2);
			 
				designator_1[i].push_back(d1);
				be_1[i].push_back(e1);
				// designator_2[i].push_back(d2);
				// be_2[i].push_back(e2);
			 
			
			}
			 
			
		}	
		
	}
	fclose(fp3);
	// fclose(fp4);
// do not modify*****************end
	ReadData();
//(begin) Do not modify, adjust binding energy for fitting parameters, only k-shell binding energy needs to be modified 
    int count1 = 0;
    int count2 = 0;
	for(int z = 1; z<=100; z++)
	{
		// number of subshell for each element does not change
		if(fNShells[z]!= be_1[z].size())
		{
			cout<<"----problem------"<<endl;
		}
		
		if(be_1[z][0]!= fParamHigh[z][1])
		{
			fParamHigh[z][1] = be_1[z][0];
			// cout<<z<<" is modified"<<endl;
			
			count1++;
		}
		// printf("%d %.9e\n", z, fParamHigh[z][1]);
		if(fParamLow[z][0]>=fParamHigh[z][1])
		{
			count2++;  // normal: low fit limit is higher than new K binding energy
		}
		else
		{
			// printf("z = %d %.9e %.9e\n", z, fParamLow[z][0], fParamHigh[z][1] );
			// cout<<"z = "<<z<<endl;
		}
	}
	cout<<"count1 = "<<count1<<endl;
	cout<<"count2 = "<<count2<<endl;
	
//(end) Do not modify, adjust binding energy for fitting parameters, only k-shell binding energy needs to be modified 
	
	
	// return;
	for(int z = 1; z<=100; z++)
	{
		// for(int i = 0; i<fCrossSection_x[z].size();i++)
		// {
			// printf("%d %.9e %.9e\n", i+1, fCrossSection_x[z][i], fCrossSection[z][i]);
		// }
		// cout<<"-----**************************************"<<endl;
		
		// for(int i = 0; i<fCrossSectionLE_x[z].size();i++)
		// {
			// printf("%d %.9e %.9e\n", i+1, fCrossSectionLE_x[z][i], fCrossSectionLE[z][i]);
		// }
	    // cout<<"***************************************number of subshells = "<<fNShells[z]<<endl;
		// for(int i = 0; i<fNShells[z]*7+1;i++)
		// {
			
			// printf("%d %.9e %.9e\n", i+1, fParamHigh[z][i], fParamLow[z][i]);
		// }
		
		char path1Format[128] = "./epics2017/phot_preliminary/pe-cs-%d.dat"; //prefix of the name of original data file
		char path1[128];
		sprintf(path1,path1Format, z);		
		plotComparison(z, path1);
	   
	}

		// PlotRatioMaxWithZ();
	

	
	
	FILE* out = fopen("ratioMaxForFittedTotalCSUsingExistingParametersFor2017_Reading.txt", "w");
	FILE* out1 = fopen("ratioMaxForFittedTotalCSUsingExistingParametersFor2017_plotting.dat", "w"); // for plotting
	int final_z3 = 0;
	double final_ratioMax3 = 0;
	
	int final_z4 = 0;
	double final_ratioMax4 = 0;
	
  
    for(int i=0; i<100;i++)
	{
		fprintf(out,"Element = %d, ratioMax3 = %.9e%% ratioMax4 = %.9e%%\n", i+1,RatioMax3[i],RatioMax4[i]);
		fprintf(out1,"%d %.9e %.9e\n", i+1,RatioMax3[i],RatioMax4[i]);
		if(abs(RatioMax3[i])> final_ratioMax3)
		{
			final_z3 = i+1;
			final_ratioMax3 = RatioMax3[i];
			
		}
		
		if(abs(RatioMax4[i])> final_ratioMax4)
		{
			final_z4 = i+1;
			final_ratioMax4 = RatioMax4[i];
			
		}
	}
	
	fprintf(out,"Amongst all the elements,\nRegion3 is:\nElement = %d, ratioMax = %.9e%% \nRegion4 is:\nElement = %d, ratioMax = %.9e%% \n", 
					    final_z3, final_ratioMax3, final_z4,final_ratioMax4 );
		
		
	fclose(out);
	fclose(out1);
	
    
	
	
	
        
	   
	// cout<<"results="<<results<<", results2="<<results2<<", results3="<<results3<<endl;
	cout<<"results1="<<results1<<endl;
	cout<<"results2="<<results2<<endl;
	// cout<<"for the following elements, all points in EPICS2014 are found in EPICS2017"
	// <<endl;
	// for(std::vector<int>::iterator it = resultsV.begin(); it!=resultsV.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	
	// cout<<"results4="<<results4<<" Only two elements do not have errors of overlapping energy pairs,\
	// namely element 1 and 2"<<endl;
	
	// cout<<"results5="<<results5<<endl;
	
	// cout<<"for the following elements, it has exceptions due to the precision:"<<endl;
	
	// for(std::vector<int>::iterator it = results5V.begin(); it!=results5V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	// cout<<"results6="<<results6<<endl;
	// cout<<"for the following elements, EPICS2017 and EPICS2014 has same repeated points:"<<endl;
	
	// for(std::vector<int>::iterator it = results6V.begin(); it!=results6V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	// cout<<"results7="<<results7<<endl;
	
	// for(std::vector<int>::iterator it = results7V.begin(); it!=results7V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	
	// cout<<"results8="<<results8<<endl;
	// cout<<"results9="<<results9<<endl;
}