// **************************************************************************
// ROOT macro: plot_total.c
// Plots the figures of total pair production cross section data. 
// There is one figure (now in png format, it can be modified) per element.
// -	cross section data of EPICS2014 (Z: 1-100) 
// -	cross section data of Livermore (Z: 1-99)   
// -	comparison of cross section data between Livermore and EPICS2014 (Z: 1-99) 
// The generated figures are saved in the corresponding directory:
// -	plots/total/comparison
// -	plots/total/epics2017
// -	plots/total/livermore

// Authors: Zhuxin LI @CENBG,France
// 2019.12.09
// **************************************************************************
TString sInt(Int_t number)
  {
    stringstream ss;
    ss << number;
	TString ts = ss.str();
    return ts;
  }
TString sDouble(Double_t number)
  {
    stringstream ss;
	ss << number;
    TString ts = ss.str();
    return ts;
  }
 	double RatioMax3[100];
	double RatioMax3_x[100];
	double RatioMax4[100];
	double RatioMax4_x[100];
 
std::vector<double> fParamHigh[101];
std::vector<double> fParamLow[101];
double fLowLimits[101];
double fHighLimits[101];


std::vector<double> designator_1[101];
std::vector<double> be_1[101];
  
  
  void ReadData()
  {
	   //**************************************************************
	// read high energy fitting parameters
	std::ostringstream ost1;
	ost1 << "highFittingParams_final.dat";
	std::ifstream fin1(ost1.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;		
		int subshell_index;
		double P[6];
		fin1 >> z >> subshell_index;
		fParamHigh[z].push_back(subshell_index);
		for(int i = 0; i<6; i++)
		{
			 fin1 >> P[i];
			 fParamHigh[z].push_back(P[i]);
			 
		}	
	 
	}
	
		
   fin1.close();
   printf("High fit parameters: %.9e\n", fParamHigh[100][188]);
	//**************************************************************
	// read low energy fitting parameters 
	std::ostringstream ost1_low;
	ost1_low << "lowFittingParams_final.dat";
	std::ifstream fin1_low(ost1_low.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;		
		int subshell_index;
		double P[6];
		fin1_low >> z >> subshell_index;
		fParamLow[z].push_back(subshell_index);
		for(int i = 0; i<6; i++)
		{
			 fin1_low >> P[i];
			 fParamLow[z].push_back(P[i]);
			 
		}		
	 
	}
	fin1_low.close();
	printf("Low fit parameters: %.9e\n", fParamLow[100][188]);
	
	//**************************************************************		  

	
	FILE*fp3 = fopen("./epics2017/phot_preliminary/bindingEnergy/binding.dat","r");	
	
	
	double d1; 
	double e1; 
    Int_t Ncols = 0;
	Int_t count = 0;
	
    for(int i=1; i<=100; ++i)  // there are 100 elements
	{
		
		
		while(1)
		{


			count++;
			Ncols = fscanf(fp3,"%lf %lf",&d1,&e1);
			
			
			// if(d1 ==d2 && d1 == -1)
			if(d1 == -1)
			{
				// cout<<"---ending---"<<i<<" "<<count<<endl;

				break;
			}
			
			else{
				
			 
				designator_1[i].push_back(d1);
				be_1[i].push_back(e1);
			 
			
			}
			 
			
		}	
		
	}
	fclose(fp3);
	
	//**************************************************************
	// read low and high limits 
	std::ostringstream ost2;
	ost2 << "low_high_limits_final.dat";
	std::ifstream fin2(ost2.str().c_str());
	for(int i = 0; i<1612; ++i) 
	{
		int z;
		double lowLimit;
		double highLimit;
		fin2 >> z >> lowLimit>>highLimit;
		fLowLimits[z] = lowLimit;
		fHighLimits[z] = highLimit;
	}
	
	printf("Low limit = %.9e High limit = %.9e\n",fLowLimits[93],fHighLimits[93]);
	fin2.close();
  }

std::map<int, double> ratioMax;
  
int results=0;
int results1=0;
std::vector<int> resultsV;
int results2=0;
int results3=0;
int results4=0;
int results5=0;
std::vector<int> results5V;
int results6=0;
std::vector<int> results6V;
int results7=0;
std::vector<int> results7V;
int results8=0;
int results9=0;
	
	
// void plotComparison(Int_t z, char*dir1, char*dir2, std::vector<double>& be_1, std::vector<double>& be_2)
void plotComparison(Int_t z, char*dir1)
{
	
	gROOT->Reset();
		
	TString S = sInt(z); 
	TString title;
	title = "EPDL2017 Photoelectric reaction: fitted (total) CS VS tabulated (total) CS for element Z = " + S;
		
		
	Int_t num1 = 0;  
	// Int_t num2 = 0; 
	
	
	Int_t ncols;
	double e, cs;
	Int_t loop = 0;
	char str[64] = "";
	cout<<"------------element "<<z<<"-------------------------"<<endl;
	FILE * fp1 = fopen(dir1,"r");
	// cout<<"yes"<<endl;
	while(1)  
	{	
		loop++;
		// if loop = 1 we read the first header line
		fgets (str, 64, fp1);
		// cout<<"yes"<<endl;
		// cout<<str<<endl;
		if(loop ==2)
		{
			// now loop = 2, we read the second header line, namely the number of points
			num1 = stoi(str);  
			// cout<<"number of total pair production points:"<<num1<<endl;
			break;
		}		
		
	}
	
	
	std::vector<double>  x_1;
	std::vector<double>  y_1; // tabulated total cs		

	std::vector<double>  y_2; // fitted total cs
	std::vector<double>  r_2; // fitted total cs ratio
	
	std::vector<double>  x_21; // fitted total cs energy below K binding energy
	std::vector<double>  x_22; // fitted total cs energy above K binding energy
	std::vector<double>  x_23; // fitted total cs energy low fit
	std::vector<double>  x_24; // fitted total cs energy high fit
	
	std::vector<double>  y_21; // fitted total cs 
	std::vector<double>  y_22; // fitted total cs above K binding energy
	std::vector<double>  y_23; // fitted total cs low fit
	std::vector<double>  y_24; // fitted total cs high fit

	std::vector<double>  r_21; // ratio below K binding energy
	std::vector<double>  r_22; // ratio above K binding energy
	std::vector<double>  r_23; // ratio low fit
	std::vector<double>  r_24; // ratio high fit
	
	
	double formerEnergy = 0.;
	for(Int_t i = 0;i<num1;++i)
	{
		// read one line energy cross section
		ncols = fscanf(fp1,"%lf %lf",&e,&cs); 
		
		// x_1[i]=e;   // keep energy
		// y_1[i]=cs;  // keep cross section
		
		
		//remove repeated points, and always keep the point with bigger cross section
		if(i && e == formerEnergy)
		{
			// printf("Repeated--%d %.9e %.9e\n", i+1,e, cs);
			y_1[y_1.size()-1] = cs;
			continue;
		}
		
		formerEnergy = e;
		
		x_1.push_back(e);  
		y_1.push_back(cs);
		// printf("--%d %.9e %.9e\n", i+1, e, cs);
		
		
	}
	fclose(fp1);
	cout<<"num1 = "<<num1<<", x_1.size() = "<<x_1.size()<<endl;
	
	// for(int i=0; i<x_1.size();i++) 
	// {
		// printf("**%d %.9e %.9e\n", i+1,x_1[i], y_1[i]);
	// }
	
	
	
	bool test1 = true;
	bool test2 = true;
	
	double ratioMax3 = 0.;
	double ratioMax4 = 0.;
	
	// high fitting parameters for total cross sections
	double PP0 = 0;
	double PP1 = 0;
	double PP2 = 0;
	double PP3 = 0;
	double PP4 = 0;
	double PP5 = 0;
	
	// low fitting parameters for total cross sections
	double PPP0 = 0;
	double PPP1 = 0;
	double PPP2 = 0;
	double PPP3 = 0;
	double PPP4 = 0;
	double PPP5 = 0;
	
	
	for(int i=0; i<be_1[z].size(); i++)
	{
		char linedata[200] = "";
		int idx = 7*i+1;
	
		PP0 += fParamHigh[z][0+idx];
		PP1 += fParamHigh[z][1+idx];
		PP2 += fParamHigh[z][2+idx];
		PP3 += fParamHigh[z][3+idx];
		PP4 += fParamHigh[z][4+idx];
		PP5 += fParamHigh[z][5+idx];
		
		
		PPP0 += fParamLow[z][0+idx];
		PPP1 += fParamLow[z][1+idx];
		PPP2 += fParamLow[z][2+idx];
		PPP3 += fParamLow[z][3+idx];
		PPP4 += fParamLow[z][4+idx];
		PPP5 += fParamLow[z][5+idx];
		
		
			
		// printf("%.9e\n", fParamHigh[z][6]);
	}
		
		
	
	for(int i=0; i<x_1.size();i++) // EPICS2017
	{
		
		double energy = x_1[i];
		double cs_0 = y_1[i];
		double cs;
		double ratio;
		
		// int idx = fNShells[z]*7 - 5;
		// energy = std::max(energy, fParamHigh[z][idx-1]);
    
		double x1 = 1.0/energy;
		double x2 = x1*x1;
		double x3 = x2*x1;
		
		
		// printf("%d %.9e %.9e \n", i, x_1[i], y_1[i]);
		
		
		// high energy parameterisation
		// if(energy >= fParamHigh[z][0]) {
		if(energy >= fHighLimits[z]) {
			
			double x4 = x2*x2;
			double x5 = x4*x1;
			
			cs = x1*(PP0 + x1*PP1
					 + x2*PP2 + x3*PP3
					 + x4*PP4+ x5*PP5);
			ratio = (cs-cs_0)/cs_0 *100;
			
			x_24.push_back(x_1[i]);	
			y_24.push_back(cs);	
			r_24.push_back(ratio);	
			
			if(abs(ratio)>abs(ratioMax4))
			{
				ratioMax4 = ratio;
			}
			// printf("-----%d %.9e %.9e %.9e %.9e\n",i, x_1[i], cs_0, cs, ratio);
					
			
			
		}
		// low energy parameterisation
		else if(energy >= fLowLimits[z]) 
		{
			double x4 = x2*x2;
			double x5 = x4*x1; //this variable usage can probably be optimized
			
			
			cs = x1*(PPP0 + x1*PPP1
			 + x2*PPP2 + x3*PPP3
			 + x4*PPP4+ x5*PPP5);
			 
			ratio = (cs-cs_0)/cs_0 *100;
			
			x_23.push_back(x_1[i]);	
			y_23.push_back(cs);
			r_23.push_back(ratio);
			// printf("*****%d %.9e %.9e %.9e %.9e\n",i, x_1[i], cs_0, cs, ratio);
			
				
			
			
			if(abs(ratio)>abs(ratioMax3))
			{
				ratioMax3 = ratio;
			}				
		}
		// Tabulated values above k-shell ionization energy
		// else if(energy >= kBE) 
		// {
			// //(begin)version for 2014
			// //search for energy in fCrossSection_x[z] vector
			// // std::vector<double>::iterator result = find(fCrossSection_x[z].begin(), fCrossSection_x[z].end(), energy);
			// // if(result != fCrossSection_x[z].end()) //if energy exist in fCrossSection_x[z]
			// // {
				// // // get its index, and use its index to get corresponding CS
				// // int index = distance(fCrossSection_x[z].begin(),result);
				// // cs = x3*fCrossSection[z][index];
				// // ratio = (cs-cs_0)/cs_0 *100;
				
				// // x_22.push_back(x_1[i]);	
				// // y_22.push_back(cs);
				// // r_22.push_back(ratio);
				
			// // }
			// // else
			// // {
				// // test1 = false;
				// // printf("-------%d %.9e\n", i+1, energy);
			// // }
			// //(end)version for 2014
			// ratio = 0.;
			// cs = y_1[i];
			// x_22.push_back(x_1[i]);	
			// y_22.push_back(cs);
			// r_22.push_back(ratio);
			
		// }
		// // Tabulated values below k-shell ionization energy
		// else
		// {
			
			// //(begin)version for 2014
			// // std::vector<double>::iterator result = find(fCrossSectionLE_x[z].begin(), fCrossSectionLE_x[z].end(), energy);
			// // if(result != fCrossSectionLE_x[z].end())
			// // {
				
				// // int index = distance(fCrossSectionLE_x[z].begin(),result);
				// // cs = x3*fCrossSectionLE[z][index];
				// // ratio = (cs-cs_0)/cs_0 *100;
				
				// // x_21.push_back(x_1[i]);	
				// // y_21.push_back(cs);
				// // r_21.push_back(ratio);
				// // // printf("---%d %.9e\n", index, energy);
				// // // printf("ttt%d %.9e %.9e %.9e\n", i+1,energy, cs, ratio);
			// // }
			// // else
			// // {
				// // test2 = false;
				// // printf("*******%d %.9e\n", i+1, energy);
			// // }
			// //(end)version for 2014
			// ratio = 0.;
			// cs = y_1[i];
			// x_21.push_back(x_1[i]);	
			// y_21.push_back(cs);
			// r_21.push_back(ratio);
			
			
		// }
		// y_2.push_back(cs);
		// r_2.push_back(ratio);
		
		
		
		
	}
	
	RatioMax3[z-1] = ratioMax3;
	RatioMax4[z-1] = ratioMax4;
	
	// return;
	if(!test1)
	{
		results1++;
	}
	if(!test2)
	{
		results2++;
	}
	// cout<<"number of points for 4 functions: "<<y_21.size()<<" "<<y_22.size()<<" "<<y_23.size()<<" "<<y_24.size()<<endl;
	
	// for(int i= 0;i<y_21.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_21[i], y_21[i], r_21[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_22.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_22[i], y_22[i], r_22[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_23.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_23[i], y_23[i], r_23[i]);
	// }
	// cout<<endl;
	// for(int i= 0;i<y_24.size();i++)
	// {
		// printf("---%d %.9e %.9e %.9e\n", i+1,x_24[i], y_24[i], r_24[i]);
	// }	
		
	TCanvas* mycanvas = new TCanvas(S,S,800, 50, 1200, 1000);
		
		mycanvas->SetGrid();
		
		gPad->SetLogx();
		gPad->SetGrid();
		
		// gPad->SetTopMargin(0.12);
		gPad->SetBottomMargin(0.15);
		gPad->SetLeftMargin(0.12);
		// gPad->SetBottomMargin(0.15);
		
		// gStyle->SetTitleW(0.9);  //per cent of the pad width
		// gStyle->SetTitleH(0.9);
		// gStyle->SetTitleFontSize(0.9);
		
		// // pad21->SetLeftMargin(0.10); //10%
		
		
		
		auto mg1 = new TMultiGraph();
		// TGraph *ratio = new TGraph(x_1.size(),x_1.data(),r_2.data());
		// TGraph *Ratio21 = new TGraph(x_21.size(),x_21.data(),r_21.data());
		// TGraph *Ratio22 = new TGraph(x_22.size(),x_22.data(),r_22.data());
		TGraph *Ratio23 = new TGraph(x_23.size(),x_23.data(),r_23.data());
		TGraph *Ratio24 = new TGraph(x_24.size(),x_24.data(),r_24.data());
		
		// Ratio21->SetMarkerStyle(24);  //red point
		// Ratio21->SetMarkerSize(0.8);
		// Ratio21->SetMarkerColor(kTeal+4);
		// Ratio21->SetLineColor(kTeal+4);
		// mg1->Add(Ratio21);	
		
		
		Ratio23->SetMarkerStyle(24);  //red point
		Ratio23->SetMarkerSize(0.8);
		Ratio23->SetMarkerColor(kBlue);
		Ratio23->SetLineColor(kBlue);
		mg1->Add(Ratio23);	
		
		
		// Ratio22->SetMarkerStyle(24);  //red point
		// Ratio22->SetMarkerSize(0.8);
		// Ratio22->SetMarkerColor(kMagenta);
		// Ratio22->SetLineColor(kMagenta);
		// mg1->Add(Ratio22);	
		
		
		
		
		Ratio24->SetMarkerStyle(24);  //red point
		Ratio24->SetMarkerSize(0.8);
		Ratio24->SetMarkerColor(kRed);
		Ratio24->SetLineColor(kRed);
		mg1->Add(Ratio24);	
		
		
		
		TString title2 = "EPDL2017: Relative difference of fitted total cross sections using new fitting parameters for element " + S;
		mg1->SetTitle(title2);
		// mg1->SetTitleSize(0.05);
		mg1->Draw("AP");
		
		mg1->GetXaxis()->SetTitle("Energy (MeV)");
		mg1->GetYaxis()->SetTitle("Relative difference (%)");
		
		// mg1->GetXaxis()->SetTitleSize(0.05);
		// mg1->GetYaxis()->SetTitleSize(0.05);
		// mg1->GetXaxis()->SetTitleOffset(1.2);
		
		// mg1->GetXaxis()->SetLabelSize(0.05);
		// mg1->GetYaxis()->SetLabelSize(0.05);
		
		
		mg1->GetXaxis()->CenterTitle();
		mg1->GetYaxis()->CenterTitle();
		mg1->GetXaxis()->SetTitleOffset(1.2);
		
		// mg1->GetXaxis()->SetLimits(1e-6,1e5);
	
		TLegend* leg1 = new TLegend(0.7,0.7,0.9,0.9);
		// leg1->SetTextAlign(12);
		// leg1->SetFillColor(0);	
		leg1->SetFillStyle(0);
		leg1->SetTextSize(0.04);
		
		
			
		
		
		// TString rname1 = "Region 1 - "+ sInt(x_21.size()) + " point(s)";
		// TString rname2 = "Region 2 - "+ sInt(x_22.size()) + " point(s)";
		// TString rname3 = "Region 3 - "+ sInt(x_23.size()) + " point(s)";
		// TString rname4 = "Region 4 - "+ sInt(x_24.size()) + " point(s)";
		TString rname3 = "Low fit";
		TString rname4 = "High fit";
		
		// leg1->AddEntry(Ratio21,rname1);
		// leg1->AddEntry(Ratio22,rname2);
		leg1->AddEntry(Ratio23,rname3);
		leg1->AddEntry(Ratio24,rname4);
		//leg1->DrawClone("Same");
		leg1->Draw();
		
		
		// // TLatex latex;
		// // latex.SetTextSize(0.05);
		// // latex.SetTextAlign(22);
		// // latex.DrawLatexNDC(0.7,0.8,"RD = #frac{EPICS2017-EPICS2014}{EPICS2014} x 100");
		
		
		// // TString Tflag = sInt(k);	
		// // Tflag += " points in comparison";
		// // latex.SetTextColor(2);
		// // latex.DrawLatexNDC(0.7,0.7,Tflag);
		
		
		
		
		// // // TLegend* leg212 =  new TLegend(0.12,0.7,0.3,0.88);
		// // // TString Tnum4 = sInt(num2);
		// // // TString name4 = Tnum4 + " points in comparison";		
		// // // leg212->AddEntry(ratio,name4);
		// // // leg212->SetTextAlign(22);
		// // // leg212->SetFillStyle(0);
		// // // leg212->Draw();
		// // // leg212->SetTextSize(0.05);
		
		
		
		
		// // //***********(begin)for the zoomin of the ratio 
		
		
		
		mycanvas->Print("plots/fittedTotalCSUsingNewParameters/fittedCS2017-"+ S +".png");
		// delete mycanvas;
		// mycanvas = NULL;
		// mycanvas->Close();
		
		
}



void plot_pe_fittedTotalCSUsingNewFitParameters2017()
{
	
	gROOT->Reset();	
	gSystem->Exec("mkdir -p plots/fittedTotalCSUsingNewParameters");
	
	ReadData();
	

	// return;
	for(int z = 1; z<=100; z++)
	{
		
		
		char path1Format[128] = "./epics2017/phot_preliminary/pe-cs-%d.dat"; //prefix of the name of original data file
		char path1[128];
		sprintf(path1,path1Format, z);	
		plotComparison(z, path1);
	   
	}

		// PlotRatioMaxWithZ();
	

	
	
	FILE* out = fopen("ratioMaxForFittedTotalCSUsingNewParams2017_Reading.txt", "w");
	FILE* out1 = fopen("ratioMaxForFittedTotalCSUsingNewParams2017_plotting.dat", "w"); // for plotting
	int final_z3 = 0;
	double final_ratioMax3 = 0;
	
	int final_z4 = 0;
	double final_ratioMax4 = 0;
	
	fprintf(out,"This file comes from plot_pe_fittedCS_2017.C\n ");
  
    for(int i=0; i<100;i++)
	{
		fprintf(out,"Element = %d, ratioMax3 = %.9e%% ratioMax4 = %.9e%%\n", i+1,RatioMax3[i],RatioMax4[i]);
		fprintf(out1,"%d %.9e %.9e\n", i+1,RatioMax3[i],RatioMax4[i]);
		if(abs(RatioMax3[i])> final_ratioMax3)
		{
			final_z3 = i+1;
			final_ratioMax3 = RatioMax3[i];
			
		}
		
		if(abs(RatioMax4[i])> final_ratioMax4)
		{
			final_z4 = i+1;
			final_ratioMax4 = RatioMax4[i];
			
		}
	}
	
	fprintf(out,"Amongst all the elements,\nRegion3 is:\nElement = %d, ratioMax = %.9e%% \nRegion4 is:\nElement = %d, ratioMax = %.9e%% \n", 
					    final_z3, final_ratioMax3, final_z4,final_ratioMax4 );
		
		
	fclose(out);
	fclose(out1);
	
    
	
	
	
        
	   
	// cout<<"results="<<results<<", results2="<<results2<<", results3="<<results3<<endl;
	cout<<"results1="<<results1<<endl;
	cout<<"results2="<<results2<<endl;
	// cout<<"for the following elements, all points in EPICS2014 are found in EPICS2017"
	// <<endl;
	// for(std::vector<int>::iterator it = resultsV.begin(); it!=resultsV.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	
	// cout<<"results4="<<results4<<" Only two elements do not have errors of overlapping energy pairs,\
	// namely element 1 and 2"<<endl;
	
	// cout<<"results5="<<results5<<endl;
	
	// cout<<"for the following elements, it has exceptions due to the precision:"<<endl;
	
	// for(std::vector<int>::iterator it = results5V.begin(); it!=results5V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	// cout<<"results6="<<results6<<endl;
	// cout<<"for the following elements, EPICS2017 and EPICS2014 has same repeated points:"<<endl;
	
	// for(std::vector<int>::iterator it = results6V.begin(); it!=results6V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	// cout<<"results7="<<results7<<endl;
	
	// for(std::vector<int>::iterator it = results7V.begin(); it!=results7V.end(); it++)
	// {
		// cout<<*it<<endl;
	// }
	
	// cout<<"results8="<<results8<<endl;
	// cout<<"results9="<<results9<<endl;
}